"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteCustomFieldResults = exports.SaveCustomFieldResult = exports.GetCustomFieldResults = exports.DeleteCustomField = exports.SaveCustomField = exports.GetCustomField = exports.GetCustomFields = exports.CustomFieldMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function CustomFieldMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.CustomFieldMSSql = CustomFieldMSSql;
function GetCustomFields(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetCustomFields')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomFields = GetCustomFields;
function GetCustomField(vcc, customFieldId, evaluationPlanId) {
    if (evaluationPlanId === void 0) { evaluationPlanId = null; }
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), customFieldId)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .execute('GetCustomField')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var customField = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(customField);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomField = GetCustomField;
function SaveCustomField(vcc, id, description, type, label, createdByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(100), id)
        .input('Description', sql.VarChar(150), description)
        .input('Type', sql.VarChar(25), type)
        .input('Label', sql.VarChar(100), label)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .execute('SaveCustomField')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCustomField = SaveCustomField;
function DeleteCustomField(vcc, customFieldId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('CustomFieldId', sql.VarChar(100), customFieldId)
        .execute('DeleteCustomField')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteCustomField = DeleteCustomField;
function GetCustomFieldResults(vcc, customFieldId, evaluationPlanId) {
    if (evaluationPlanId === void 0) { evaluationPlanId = null; }
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('CustomFieldId', sql.VarChar(100), customFieldId)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .execute('GetCustomFieldResults')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomFieldResults = GetCustomFieldResults;
function SaveCustomFieldResult(vcc, id, description, order, customFieldId, createdByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(100), id)
        .input('Description', sql.VarChar(150), description)
        .input('Order', sql.Int, order)
        .input('CustomFieldId', sql.VarChar(150), customFieldId)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .execute('SaveCustomFieldResult')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCustomFieldResult = SaveCustomFieldResult;
function DeleteCustomFieldResults(vcc, customFieldId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('CustomFieldId', sql.VarChar(100), customFieldId)
        .execute('DeleteCustomFieldResults')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteCustomFieldResults = DeleteCustomFieldResults;
