"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteGenericQuery = exports.SaveGenericQuery = exports.GetGenericQueryUsage = exports.GetGenericQueryValidate = exports.GetGenericQuery = exports.GetGenericQueries = exports.GenericQueryMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function GenericQueryMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.GenericQueryMSSql = GenericQueryMSSql;
function GetGenericQueries(vcc, search_criteria, search_expression, startDate, endDate) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('StartDate', sql.VarChar(10), startDate)
        .input('EndDate', sql.VarChar(10), endDate)
        .execute('GetGenericQueries')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGenericQueries = GetGenericQueries;
function GetGenericQuery(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetGenericQuery')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var genericQuery = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(genericQuery);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGenericQuery = GetGenericQuery;
function GetGenericQueryValidate(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetGenericQueryValidate')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var genericQuery = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(genericQuery);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGenericQueryValidate = GetGenericQueryValidate;
function GetGenericQueryUsage(vcc, QueryName) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('QueryName', sql.VarChar(50), QueryName)
        .execute('GetGenericQueryUsage')
        .then(function (recordsets) {
        var abilitiesRecordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var templateThemeItemsRecordset = (recordsets && recordsets.length > 0 ? recordsets[1] : null);
        var evaluationPlanRecordeset = (recordsets && recordsets.length > 0 ? recordsets[2] : null);
        var abilities = (abilitiesRecordset && abilitiesRecordset.length > 0 ? abilitiesRecordset : null);
        var templateThemeItems = (templateThemeItemsRecordset && templateThemeItemsRecordset.length > 0 ? templateThemeItemsRecordset[0] : null);
        var evaluationsPlan = (evaluationPlanRecordeset && evaluationPlanRecordeset.length > 0 ? evaluationPlanRecordeset : null);
        var result = {
            abilities: abilities,
            templateThemeItems: templateThemeItems,
            evaluationsPlan: evaluationsPlan
        };
        return Promise.resolve(result);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[GenericQueryMSSql:GetGenericQueryUsage] Getting error:', errorMsg, 'vcc:', vcc, 'queryName:', QueryName);
        return Promise.reject(err);
    });
}
exports.GetGenericQueryUsage = GetGenericQueryUsage;
function SaveGenericQuery(vcc, name, description, query, rawData, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId, version) {
    if (version === void 0) { version = 1; }
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(100), name)
        .input('Description', sql.VarChar(500), description)
        .input('Query', sql.VarChar(sql.MAX), query)
        .input('RawData', sql.VarChar(sql.MAX), rawData)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .input('Version', sql.SmallInt, version)
        .execute('SaveGenericQuery')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveGenericQuery = SaveGenericQuery;
function DeleteGenericQuery(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteGenericQuery')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteGenericQuery = DeleteGenericQuery;
