"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInteractionByTypeAndId = exports.getInteractions = void 0;
var callDB = require("./call");
var interaction_model_1 = require("../../../models/interaction.model");
var log4js = require("log4js");
var logger = log4js.getLogger('ServerScripts');
function getInteractions(vcc, limit, params) {
    return new Promise(function (resolve, reject) {
        var promises = [];
        logger.info('[HistoricalData::getInteractions] Searched types: ' + params.types.length);
        if (params.types && params.types.length > 0) {
            if (params.types.includes('CALL')) {
                logger.info('[HistoricalData::getInteractions]  Added interactions search "Call" for vcc ' + vcc);
                promises.push(callDB.getInteractionsRefactor(vcc, limit, params));
            }
        }
        var res = new interaction_model_1.InteractionView;
        res.totalRecords = 0;
        res.totalTime = 0;
        res.roundedTime = 0;
        res.interactions = [];
        logger.info('[HistoricalData::getInteractions] Request the added promises.');
        Promise.all(promises).then(function (results) {
            logger.info('[HistoricalData::getInteractions] Request OK');
            for (var _i = 0, results_1 = results; _i < results_1.length; _i++) {
                var interactionView = results_1[_i];
                res.totalRecords += interactionView['totalRecords'];
                res.totalTime += interactionView['totalTime'];
                res.roundedTime += interactionView['roundedTime'];
                for (var _a = 0, _b = interactionView['interactions']; _a < _b.length; _a++) {
                    var i = _b[_a];
                    res.interactions.push(i);
                }
            }
            resolve(res);
        })
            .catch(function (error) {
            logger.error('[HistoricalData::getInteractions] Error during search in HistoricalDara: ' + error);
            reject(error);
        });
    });
}
exports.getInteractions = getInteractions;
function getInteractionByTypeAndId(vcc, type, id, loadContent) {
    if (loadContent === void 0) { loadContent = false; }
    switch (type) {
        case 'CALL': return callDB.getInteractionById(vcc, id, loadContent);
        default:
            return new Promise(function (resolve, reject) {
                reject('Interaction Type not valid or not found: ' + type);
            });
    }
}
exports.getInteractionByTypeAndId = getInteractionByTypeAndId;
exports.default = {
    getInteractionByTypeAndId: getInteractionByTypeAndId,
    getInteractions: getInteractions
};
