"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BulkAsyncInteractionTemp = exports.AsyncInteractionMSSql = void 0;
var log4js = require("log4js");
var moment = require("moment");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function AsyncInteractionMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AsyncInteractionMSSql = AsyncInteractionMSSql;
function BulkAsyncInteractionTemp(vcc, processId, asyncInteractionArray, userId) {
    var logLabel = '[vcc::processId => ' + vcc + '::' + processId + ']: ';
    var tableAsyncInteraction = new sql.Table('#AsyncInteractionTemp');
    tableAsyncInteraction.create = true;
    var tableProcessAsyncInteraction = new sql.Table('#ProcessAsyncInteractionTemp');
    tableProcessAsyncInteraction.create = true;
    tableAsyncInteraction.columns.add('VCC', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('ID', sql.VarChar(250), { nullable: true });
    tableAsyncInteraction.columns.add('Folder', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('ExportedContent', sql.VarChar(250), { nullable: true });
    tableAsyncInteraction.columns.add('Channel', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('Campaign', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('StartDate', sql.DateTime, { nullable: true });
    tableAsyncInteraction.columns.add('EndDate', sql.DateTime, { nullable: true });
    tableAsyncInteraction.columns.add('Account', sql.VarChar(250), { nullable: true });
    tableAsyncInteraction.columns.add('Initiative', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('ContactName', sql.VarChar(300), { nullable: true });
    tableAsyncInteraction.columns.add('FirstTakedAgent', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('LastTakedAgent', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('IsTaken', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('TotalMessages', sql.Int, { nullable: true });
    tableAsyncInteraction.columns.add('DurationTime', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('AttentionTime', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('DispositionCode', sql.VarChar(sql.MAX), { nullable: true });
    tableAsyncInteraction.columns.add('DispositionTreePath', sql.VarChar(sql.MAX), { nullable: true });
    tableAsyncInteraction.columns.add('RawChar', sql.Int, { nullable: true });
    tableAsyncInteraction.columns.add('CreatedDate', sql.DateTime, { nullable: true });
    tableAsyncInteraction.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('LastModifiedDate', sql.DateTime, { nullable: true });
    tableAsyncInteraction.columns.add('LastModifiedByUserId', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('VCC', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('InteractionType', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('Campaign', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('InteractionId', sql.VarChar(250), { nullable: true });
    tableProcessAsyncInteraction.columns.add('EndDate', sql.DateTime, { nullable: true });
    tableProcessAsyncInteraction.columns.add('FormattedEndDate', sql.VarChar(100), { nullable: true });
    tableProcessAsyncInteraction.columns.add('CreatedDate', sql.DateTime, { nullable: true });
    tableProcessAsyncInteraction.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('LastModifiedDate', sql.DateTime, { nullable: true });
    tableProcessAsyncInteraction.columns.add('LastModifiedByUserId', sql.VarChar(50), { nullable: true });
    asyncInteractionArray.map(function (ai) {
        tableAsyncInteraction.rows.add(vcc, ai.id ? ai.id : null, ai.folder ? ai.folder : null, ai.exportedContent ? ai.exportedContent : null, ai.channel ? ai.channel : null, ai.campaign ? ai.campaign : null, ai.startDate ? ai.startDate : null, ai.endDate ? ai.endDate : null, ai.account ? ai.account : null, ai.initiative ? ai.initiative : null, ai.contactName ? ai.contactName : null, ai.firstTakedAgent ? ai.firstTakedAgent : null, ai.lastTakedAgent ? ai.lastTakedAgent : null, ai.isTaken ? ai.isTaken : null, ai.totalMessages ? ai.totalMessages : null, ai.durationTime ? ai.durationTime : null, ai.attentionTime ? ai.attentionTime : null, ai.dispositionCode ? ai.dispositionCode : null, ai.dispositionTreePath ? ai.dispositionTreePath : null, ai.rawChar ? ai.rawChar : null, moment().utc().toDate(), userId, moment().utc().toDate(), userId);
        tableProcessAsyncInteraction.rows.add(vcc, processId ? processId : null, ai.channel ? ai.channel : null, ai.campaign ? ai.campaign : null, ai.id ? ai.id : null, ai.endDate ? ai.endDate : null, ai.folder ? ai.folder : null, moment().utc().toDate(), userId, moment().utc().toDate(), userId);
    });
    logger.info('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'Temporary tables filled (AsyncInteractionTemp, ProcessAsyncInteractionTemp)');
    var genericResult = {};
    var transaction = new sql.Transaction(sqlConn);
    var request;
    return transaction.begin()
        .then(function (result) {
        logger.info('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'Creating transaction');
        return new sql.Request(transaction);
    })
        .then(function (result) {
        request = result;
        return request.bulk(tableAsyncInteraction)
            .then(function (rowCount) {
            logger.info('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'Inserted ' + rowCount + ' records to #AsyncInteractionTemp temporary Table');
            return true;
        })
            .catch(function (err) {
            logger.error('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'Bulk failed: ' + err.message);
            throw err;
        });
    })
        .then(function (result) {
        return request.bulk(tableProcessAsyncInteraction)
            .then(function (rowCount) {
            logger.info('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'Inserted ' + rowCount + ' records to #ProcessAsyncInteractionTemp temporary Table');
            return true;
        })
            .catch(function (err) {
            logger.error('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'Bulk failed: ' + err.message);
            throw err;
        });
    })
        .then(function (result) {
        return request.execute('BulkAsyncInteraction')
            .then(function (recordset) {
            logger.info('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'BulkAsyncInteraction OK');
            return Promise.resolve(true);
        })
            .catch(function (err) {
            logger.error('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'BulkAsyncInteraction failed: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then(function (result) {
        logger.info('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'Committing transaction');
        return transaction.commit();
    })
        .then(function (result) {
        genericResult.status = true;
        genericResult.description = 'OK';
        return Promise.resolve(genericResult);
    })
        .catch(function (err) {
        transaction.rollback();
        logger.error('[AsyncInteractionMSSql::BulkAsyncInteractionTemp] ' + logLabel + 'Bulk error failed: ' + err.message);
        genericResult.status = false;
        genericResult.description = 'SERVER_ERROR_UNKNOWN_MESSAGE';
        genericResult.error = typeof err === 'string' ? err : err.message;
        return Promise.resolve(genericResult);
    });
}
exports.BulkAsyncInteractionTemp = BulkAsyncInteractionTemp;
