"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteCustomTemplateTheme = exports.DeleteCustomTemplate = exports.SaveCustomTemplateThemeItem = exports.SaveCustomTemplateTheme = exports.SaveCustomTemplate = exports.GetCustomTemplateThemeItems = exports.GetCustomTemplateThemes = exports.GetCustomTemplateByName = exports.GetCustomTemplate = exports.GetCustomTemplates = exports.CustomTemplateMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function CustomTemplateMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.CustomTemplateMSSql = CustomTemplateMSSql;
function GetCustomTemplates(vcc, type, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.VarChar(50), type)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetTemplates')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomTemplates = GetCustomTemplates;
function GetCustomTemplate(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetTemplate')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var customTemplate = (recordset && recordset.length > 0 ? recordset[0] : null);
        if (customTemplate) {
            recordset = (recordsets && recordsets.length > 1 ? recordsets[1] : null);
            customTemplate.templateThemes = (recordset ? recordset : null);
        }
        return Promise.resolve(customTemplate);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomTemplate = GetCustomTemplate;
function GetCustomTemplateByName(vcc, type, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.VarChar(50), type)
        .input('Name', sql.VarChar(50), name)
        .execute('GetTemplateByName')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var customTemplate = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(customTemplate);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomTemplateByName = GetCustomTemplateByName;
function GetCustomTemplateThemes(vcc, templateId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TemplateId', sql.VarChar(50), templateId)
        .execute('GetTemplateThemes')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomTemplateThemes = GetCustomTemplateThemes;
function GetCustomTemplateThemeItems(vcc, templateId, templateThemeId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TemplateId', sql.VarChar(50), templateId)
        .input('TemplateThemeId', sql.VarChar(50), templateThemeId)
        .execute('GetTemplateThemeItems')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomTemplateThemeItems = GetCustomTemplateThemeItems;
function SaveCustomTemplate(vcc, name, type, description, template, category, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('Type', sql.VarChar(50), type)
        .input('Description', sql.VarChar(50), description)
        .input('Template', sql.VarChar(50), template)
        .input('Category', sql.VarChar(50), category)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveTemplate')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCustomTemplate = SaveCustomTemplate;
function SaveCustomTemplateTheme(vcc, customTemplateId, id, order, description, category1, category2, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TemplateId', sql.VarChar(50), customTemplateId)
        .input('ID', sql.VarChar(50), id)
        .input('Order', sql.Int, order)
        .input('Description', sql.VarChar(50), description)
        .input('Category1', sql.VarChar(50), category1)
        .input('Category2', sql.VarChar(50), category2)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveTemplateTheme')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCustomTemplateTheme = SaveCustomTemplateTheme;
function SaveCustomTemplateThemeItem(vcc, customTemplateId, customTemplateThemeId, id, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TemplateId', sql.VarChar(50), customTemplateId)
        .input('TemplateThemeId', sql.VarChar(50), customTemplateThemeId)
        .input('ID', sql.VarChar(50), id)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveTemplateThemeItem')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCustomTemplateThemeItem = SaveCustomTemplateThemeItem;
function DeleteCustomTemplate(vcc, name, type) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteTemplate')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteCustomTemplate = DeleteCustomTemplate;
function DeleteCustomTemplateTheme(vcc, name, type) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteTemplateTheme')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteCustomTemplateTheme = DeleteCustomTemplateTheme;
