"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllTags = exports.GetAllTagsInteraction = exports.DeleteTag = exports.SaveTag = exports.GetUniqueTags = exports.GetTags = exports.TagMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function TagMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.TagMSSql = TagMSSql;
function GetTags(vcc, audioId, section, part) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .execute('GetTags')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTags = GetTags;
function GetUniqueTags(vcc) {
    logger.debug('[GetUniqueTags] Calling SP GetUniqueTags for vcc ' + vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetUniqueTags')
        .then(function (recordsets) {
        logger.debug('[GetUniqueTags] Getting result from SP.');
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[GetUniqueTags] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUniqueTags = GetUniqueTags;
function SaveTag(vcc, audioId, section, part, tag, currentTime, comment, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .input('Tag', sql.VarChar(50), tag)
        .input('CurrentTime', sql.Float, currentTime)
        .input('Comment', sql.VarChar(200), comment)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveTag')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveTag = SaveTag;
function DeleteTag(vcc, audioId, section, part, tag) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .input('Tag', sql.VarChar(50), tag)
        .execute('DeleteTag')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteTag = DeleteTag;
function GetAllTagsInteraction(vcc, audioId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AudioId', sql.VarChar(50), audioId)
        .execute('GetAllTagsInteraction')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllTagsInteraction = GetAllTagsInteraction;
function GetAllTags(vcc) {
    logger.debug('[GetAllTags] Calling SP GetAllTags for vcc ' + vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetAllTags')
        .then(function (recordsets) {
        logger.debug('[GetAllTags] Getting result from SP.');
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error executing \'' + err.procName + '\': ' : 'Error on reques: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[GetAllTags] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllTags = GetAllTags;
