"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioTransferImplementation = void 0;
var log4js = require("log4js");
var fs = require("fs");
var request = require("request");
var child_process = require("child_process");
var mssql_service_1 = require("../db/mssql.service");
var integration_config_1 = require("../config/integration.config");
var config_1 = require("../config/config");
var file_manager_1 = require("../shared/file-manager");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var exec = child_process.exec;
var billing_implementation_1 = require("../implementation/billing.implementation");
var audio_service_1 = require("../services/audio.service");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'AudioTransferImplementation';
var AudioTransferImplementation = (function () {
    function AudioTransferImplementation() {
    }
    AudioTransferImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_AUDIO_TRANSFER;
        var executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_AUDIO_TRANSFER)
            .then(function (result) {
            return redis_service_1.Redis.LRange('Audios', 0, ratio - 1);
        }).then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.AudioTransferExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('Audios', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[AudioTransferImplementation::Execute] executionCode:', executionCode, 'getting error results:', errorMessage);
            return Promise.resolve(false);
        });
    };
    AudioTransferImplementation.prototype.AudioTransferExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var batchTypeName = '';
        var batchTypeId = '';
        return redis_service_1.Redis.RPopLPush('Audios', 'AudiosExecute')
            .then(function (queueId) {
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (processInteractionResult) {
            if (processInteractionResult) {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1');
                processInteraction = processInteractionResult;
                var baseFolder = '';
                if (processInteraction.interactionType === 'CALL') {
                    baseFolder = (processInteraction.environmentType === 'allegro' || processInteraction.environmentType === 'i6') ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
                }
                else {
                    baseFolder = config_1.TRANSCRIPTION_FOLDER;
                }
                return file_manager_1.CreateFolder(baseFolder + '/' + processInteraction.vcc);
            }
            else {
                if (currentQueueId)
                    logger.warn('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return null;
            }
        })
            .then(function (_) {
            if (processInteraction && (processInteraction.environmentType === 'allegro' || processInteraction.environmentType === 'i6')) {
                if (processInteraction.interactionType === 'CALL') {
                    return _this.GetIntegratedAudioFile(processInteraction, currentQueueId, executionCode);
                }
                else {
                    return _this.GetI6TextFile(processInteraction, currentQueueId, executionCode);
                }
            }
            else if (processInteraction && processInteraction.environmentType === 'ftp') {
                return _this.GetFTPAudioFile(processInteraction, currentQueueId, executionCode);
            }
            else if (processInteraction && processInteraction.environmentType === 'sftp') {
                return _this.GetSFTPAudioFile(processInteraction, currentQueueId, executionCode);
            }
            return null;
        }).then(function (result) {
            if (processInteraction.status
                && processInteraction.status === 1
                && processInteraction.statusDescription
                && processInteraction.statusDescription === 'pendingAnalyze') {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TO_MEDIA_ANALIZE');
                return _this.GetAudioInfo(processInteraction, currentQueueId, executionCode);
            }
            return null;
        }).then(function (result) {
            return mssql_service_1.MSSql.GetEvaluationPlanByProcessId(vcc, processId);
        })
            .then(function (result) {
            if (result !== null) {
                batchTypeName = 'EVALUATION_PLAN';
                batchTypeId = result.id;
            }
            else {
                batchTypeName = 'TRANSCRIPTION';
                batchTypeId = processInteraction.processId;
            }
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        }).then(function (result) {
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result && processInteraction && processInteraction.status === 1) {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                if (processInteraction.statusDescription === 'pendingUploadRawAudio') {
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('DiarizationConvert', currentQueueId));
                }
                else if (processInteraction.statusDescription === 'pendingUpload') {
                    promiseArray.push(redis_service_1.Redis.CreateQueueR('Transcriptions', currentQueueId));
                }
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.errorMessage === 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND') {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId, 0, 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_AUDIO_TRANSFER;
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_AUDIO_TRANSFER) {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            var promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_AUDIO_TRANSFER) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchTypeId, batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    AudioTransferImplementation.prototype.GetIntegratedAudioFile = function (interaction, currentQueueId, executionCode) {
        var localFilePath = '';
        var localFileName = '';
        return audio_service_1.GetAudioFileFromIntegrated(interaction.vcc, interaction.environmentType, interaction.id, interaction.section, interaction.part, null, interaction.format, 'REMOTE')
            .catch(function (err) {
            throw err && err.extraContent ? err.extraContent : 'GET_AUDIO_REQUEST_FAILED';
        })
            .then(function (result) {
            localFileName += result.audioFile.split('/').pop();
            localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise(function (resolve, reject) {
                fs.open(localFilePath, 'w', function (err, fd) {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                        if (err) {
                            logger.info('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                            reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, function () {
                            resolve('OK');
                        });
                    });
                });
            });
        })
            .then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingUpload';
                interaction['retry'] = 0;
                interaction['filePath'] = localFilePath;
                interaction['fileName'] = localFileName;
            }
            else {
                logger.error('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetIntegratedAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetAllegroAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    AudioTransferImplementation.prototype.GetI6TextFile = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        return Promise.resolve(true)
            .then(function (result) {
            var ip = integration_config_1.I6_TEXT_HOST;
            var port = integration_config_1.I6_TEXT_PORT;
            var urlI6Repository = 'http://';
            urlI6Repository += ip;
            urlI6Repository += ':';
            urlI6Repository += port;
            urlI6Repository += '/';
            urlI6Repository += interaction.folder;
            urlI6Repository += '/';
            urlI6Repository += interaction.vcc;
            urlI6Repository += '/';
            urlI6Repository += interaction.campaign;
            urlI6Repository += '/';
            urlI6Repository += interaction.interactionType;
            urlI6Repository += '/data/';
            var localPath = config_1.TRANSCRIPTION_FOLDER;
            localPath += '/';
            localPath += interaction.vcc;
            localPath += '/';
            var processInteractionId = JSON.parse(JSON.stringify(interaction.id));
            var cleanedProcessInteractionId = processInteractionId.split(':').join('_');
            var localJsonFileName = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + cleanedProcessInteractionId + '.json';
            var localViewFileName = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + cleanedProcessInteractionId;
            var remoteJsonFileName = cleanedProcessInteractionId + '.json';
            var remoteViewFileName = '';
            if (interaction.interactionType === 'MAIL') {
                localViewFileName += '.eml';
                remoteViewFileName = cleanedProcessInteractionId + '/' + cleanedProcessInteractionId + '_' + interaction.totalMessages + '.eml';
            }
            else {
                localViewFileName += '.html';
                remoteViewFileName = cleanedProcessInteractionId + '.html';
            }
            var localJsonFilePath = localPath + localJsonFileName;
            var localViewFilePath = localPath + localViewFileName;
            var remoteJsonFilePath = urlI6Repository + remoteJsonFileName;
            var remoteViewFilePath = urlI6Repository + remoteViewFileName;
            return promises_shared_1.DummyPromise()
                .then(function (result) {
                var promiseArray = [];
                promiseArray.push(_this.GetI6TextFileFromUrl(executionCode, currentQueueId, localJsonFilePath, remoteJsonFilePath));
                return Promise.all(promiseArray);
            })
                .then(function (result) {
                if (result.filter(function (r) { return r.status === true; }).length === 1) {
                    interaction['status'] = 1;
                    interaction['statusDescription'] = 'pendingUpload';
                    interaction['retry'] = 0;
                    interaction['fileName'] = localJsonFileName;
                    interaction['viewFileName'] = localViewFileName;
                    interaction['errorFunction'] = null;
                    interaction['errorMessage'] = null;
                    interaction['errorDetail'] = null;
                }
                else {
                    logger.error('[AudioTransferImplementation::GetI6TextFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                    interaction['status'] = -1;
                    interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                    interaction['errorFunction'] = 'GetI6TextFile';
                    interaction['errorMessage'] = 'FILES_DOWNLOAD_FAILED';
                    interaction['errorDetail'] = 'FILES_DOWNLOAD_FAILED';
                }
                return Promise.resolve(interaction);
            })
                .catch(function (err) {
                var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
                logger.error('[GetAllegroAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetI6TextFile';
                interaction['errorMessage'] = errorMessage;
                interaction['errorDetail'] = JSON.stringify(err);
                return Promise.resolve(interaction);
            });
        });
    };
    AudioTransferImplementation.prototype.GetFileNamePathFromInteraction = function (interaction, locationType, fileType, message) {
        var fileNamePath = '';
        var localPath = '';
        var urlI6Repository = '';
        if (locationType === 'local') {
            localPath = config_1.TRANSCRIPTION_FOLDER;
            localPath += '/';
            localPath += interaction.vcc;
            localPath += '/';
            fileNamePath = urlI6Repository;
        }
        else {
            var ip = integration_config_1.I6_TEXT_HOST;
            var port = integration_config_1.I6_TEXT_PORT;
            urlI6Repository = 'http://';
            urlI6Repository += ip;
            urlI6Repository += ':';
            urlI6Repository += port;
            urlI6Repository += '/';
            urlI6Repository += interaction.folder;
            urlI6Repository += '/';
            urlI6Repository += interaction.vcc;
            urlI6Repository += '/';
            urlI6Repository += interaction.campaign;
            urlI6Repository += '/';
            urlI6Repository += interaction.interactionType;
            urlI6Repository += '/data/';
            fileNamePath = urlI6Repository;
        }
        fileNamePath += this.GetFileNameFromInteraction(interaction, locationType, fileType, message);
        return fileNamePath;
    };
    AudioTransferImplementation.prototype.GetFileNameFromInteraction = function (interaction, locationType, fileType, message) {
        var interactionId = interaction.id ? interaction.id : interaction.interactionId;
        var fileName = '';
        var processInteractionId = JSON.parse(JSON.stringify(interactionId));
        var cleanedProcessInteractionId = processInteractionId.split(':').join('_');
        var localViewFileName = '';
        var localJsonFileName = '';
        var remoteViewFileName = '';
        var remoteJsonFileName = '';
        if (locationType === 'local') {
            localViewFileName = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + cleanedProcessInteractionId;
            localJsonFileName = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + cleanedProcessInteractionId + '.json';
        }
        else {
            remoteViewFileName = '';
            remoteJsonFileName = cleanedProcessInteractionId + '.json';
        }
        if (interaction.interactionType === 'MAIL') {
            localViewFileName += '.eml';
            remoteViewFileName = cleanedProcessInteractionId + '/' + cleanedProcessInteractionId + '_';
            if (message && !isNaN(message)) {
                remoteViewFileName += message;
            }
            else {
                remoteViewFileName += interaction.totalMessages;
            }
            remoteViewFileName += '.eml';
        }
        else {
            localViewFileName += '.html';
            remoteViewFileName = cleanedProcessInteractionId + '.html';
        }
        if (locationType === 'local' && fileType === 'view')
            fileName = localViewFileName;
        if (locationType === 'local' && fileType === 'json')
            fileName = localJsonFileName;
        if (locationType === 'remote' && fileType === 'view')
            fileName = remoteViewFileName;
        if (locationType === 'remote' && fileType === 'json')
            fileName = remoteJsonFileName;
        return fileName;
    };
    AudioTransferImplementation.prototype.GetI6TextFileFromUrl = function (executionCode, currentQueueId, localI6TextFileName, remoteI6TextFilePath) {
        logger.info('[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' url:', remoteI6TextFilePath);
        var file;
        var genericResult = {};
        return new Promise(function (resolve, reject) {
            request.get(remoteI6TextFilePath)
                .on('response', function (response) {
                if (response && response.headers && response.statusCode) {
                    if (response.statusCode === 200) {
                        file = fs.createWriteStream(localI6TextFileName);
                        ;
                        response.pipe(file);
                        file.on('finish', function () {
                            file.close();
                            genericResult.status = true;
                            genericResult.result = 'DOWNLOAD_SUCCESSFULL';
                            return resolve(genericResult);
                        });
                        file.on('error', function () {
                            file.close();
                            genericResult.status = false;
                            genericResult.result = 'DOWNLOAD_FAILED';
                            return resolve(genericResult);
                        });
                    }
                    else {
                        genericResult.status = false;
                        genericResult.result = 'GET_TEXT_REQUEST_FAILED';
                        return resolve(genericResult);
                    }
                }
                else {
                    logger.error('[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage: NO_RESPONSE_HEADERS');
                    genericResult.status = false;
                    genericResult.error = 'NO_RESPONSE_HEADERS';
                    return resolve(genericResult);
                }
            })
                .on('error', function (err) {
                if (err !== undefined && err !== null) {
                    var errorMessage = typeof err === 'string' ? err : err.message;
                    logger.error('[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'cannot download audio from: ', remoteI6TextFilePath, 'getting errorMessage:', errorMessage);
                    genericResult.status = false;
                    genericResult.error = err.message;
                    return resolve(genericResult);
                }
            });
        });
    };
    AudioTransferImplementation.prototype.GetFTPAudioFile = function (interaction, currentQueueId, executionCode) {
        var localFilePath = '';
        var localFileName = '';
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (interaction.id && interaction.id.length > 0) {
                localFileName = interaction.id;
                localFileName += '_';
                localFileName += interaction.section && interaction.section > 0 ? interaction.section : '0';
                localFileName += '_';
                localFileName += interaction.part && interaction.part > 0 ? interaction.part : '0';
            }
            var audioData = {
                audioId: interaction.id,
                relativePath: interaction.relativePath,
                fileName: interaction.fileName,
            };
            var environment = {};
            environment.apiHost = interaction.apiHost;
            environment.apiPort = interaction.apiPort;
            environment.apiUser = interaction.apiUser;
            environment.apiPassword = interaction.apiPassword;
            environment.ftpDirectory = interaction.ftpDirectory;
            return audio_service_1.GetAudioFileFromFtp(interaction.vcc, audioData, environment)
                .catch(function (err) {
                throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_FTP';
            });
        })
            .then(function (result) {
            localFileName += '.' + result.audioFile.split('.').pop();
            localFilePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise(function (resolve, reject) {
                fs.open(localFilePath, 'w', function (err, fd) {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                        if (err) {
                            logger.info('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                            reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, function () {
                            resolve('OK');
                        });
                    });
                });
            });
        })
            .then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingAnalyze';
                interaction['retry'] = 0;
                interaction['fileName'] = localFileName;
                interaction['filePath'] = localFilePath;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetFTPAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetFTPAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    AudioTransferImplementation.prototype.GetSFTPAudioFile = function (interaction, currentQueueId, executionCode) {
        var localFilePath = '';
        var localFileName = '';
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (interaction.id && interaction.id.length > 0) {
                localFileName = interaction.id;
                localFileName += '_';
                localFileName += interaction.section && interaction.section > 0 ? interaction.section : '0';
                localFileName += '_';
                localFileName += interaction.part && interaction.part > 0 ? interaction.part : '0';
            }
            var audioData = {
                audioId: interaction.id,
                relativePath: interaction.relativePath,
                fileName: interaction.fileName,
            };
            var environment = {};
            environment.apiHost = interaction.apiHost;
            environment.apiPort = interaction.apiPort;
            environment.apiUser = interaction.apiUser;
            environment.apiPassword = interaction.apiPassword;
            environment.ftpDirectory = interaction.ftpDirectory;
            return audio_service_1.GetAudioFileFromSftp(interaction.vcc, audioData, environment)
                .catch(function (err) {
                throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_SFTP';
            });
        })
            .then(function (result) {
            localFileName += '.' + result.audioFile.split('.').pop();
            localFilePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise(function (resolve, reject) {
                fs.open(localFilePath, 'w', function (err, fd) {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                        if (err) {
                            logger.info('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                            reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, function () {
                            resolve('OK');
                        });
                    });
                });
            });
        })
            .then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingAnalyze';
                interaction['retry'] = 0;
                interaction['fileName'] = localFileName;
                interaction['filePath'] = localFilePath;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetSFTPAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetSFTPAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    AudioTransferImplementation.prototype.GetAudioInfo = function (interaction, currentQueueId, executionCode) {
        var needToDiarizate = false;
        var tmpPath = '';
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            var cmd = '"' + config_1.MEDIAINFO_FOLDER_PATH + 'mediainfo" --Output=JSON "' + interaction.filePath + '"';
            logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd1:', cmd);
            return new Promise(function (resolve, reject) {
                try {
                    exec(cmd, function (err, stdout, stderr) {
                        logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting stderr:', stderr);
                        if (err) {
                            logger.error('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', err);
                            reject(err);
                        }
                        else {
                            var res = typeof stdout === 'string' ? JSON.parse(stdout) : stdout;
                            resolve(res.media);
                        }
                    });
                }
                catch (e) {
                    reject(e);
                }
            });
        }).then(function (result) {
            if (result && result.track) {
                var generalInfo = result.track.find(function (t) { return t['@type'] === 'General'; });
                var audioInfo = result.track.find(function (t) { return t['@type'] === 'Audio'; });
                var minDurationToDiarizate = 60;
                var maxDurationToDiarizate = 1800;
                if (generalInfo && audioInfo) {
                    interaction.format = generalInfo.FileExtension;
                    interaction.channels = Number(audioInfo.Channels) === 1 ? 'mono' : 'stereo';
                    interaction.mediaInfoTime = audioInfo.Duration;
                    if (generalInfo.FileExtension === 'wav'
                        && Number(audioInfo.Channels) === 1
                        && Number(audioInfo.Duration) >= minDurationToDiarizate
                        && Number(audioInfo.Duration) <= maxDurationToDiarizate) {
                        needToDiarizate = true;
                        tmpPath = interaction.filePath + '-TMP';
                        return file_manager_1.RenameFile(interaction.filePath, tmpPath);
                    }
                    else {
                        return Promise.resolve(true);
                    }
                }
                else {
                    interaction['retry'] = config_1.RETRY_AUDIO_TRANSFER;
                    throw 'SERVER_ERROR_CANNOT_GET_AUDIO_MEDIAINFO';
                }
            }
            else {
                interaction['retry'] = config_1.RETRY_AUDIO_TRANSFER;
                throw 'SERVER_ERROR_CANNOT_GET_AUDIO_MEDIAINFO';
            }
        })
            .then(function (result) {
            if (needToDiarizate) {
                return mssql_service_1.MSSql.GetNLUByAppId(interaction.vcc, interaction.vcc + '::diarization-' + interaction.language);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            if (needToDiarizate) {
                if (!result
                    ||
                        (result && !result.lastTrainedDateTime))
                    throw 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND';
                var cmd_1 = config_1.FFMPEG_FOLDER_PATH + 'ffmpeg -i "' + tmpPath + '" -vn -ar 8000 -ac 1 -acodec pcm_s16le -f wav -fflags +bitexact -flags:a +bitexact -map_metadata -1 "' + interaction.filePath + '"';
                logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd2:', cmd_1);
                return new Promise(function (resolve, reject) {
                    try {
                        exec(cmd_1, function (err, stdout, stderr) {
                            if (err) {
                                reject(err);
                            }
                            else {
                                resolve(stdout.media);
                            }
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            }
            else {
                return Promise.resolve(null);
            }
        }).then(function (result) {
            if (needToDiarizate) {
                return file_manager_1.DeleteFile(tmpPath);
            }
            else {
                return Promise.resolve(null);
            }
        }).then(function (result) {
            if (needToDiarizate) {
                interaction['statusDescription'] = 'pendingUploadRawAudio';
            }
            else {
                interaction['statusDescription'] = 'pendingUpload';
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetAudioInfo';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            if (errorMessage === 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND')
                file_manager_1.RenameFile(tmpPath, interaction.filePath);
            return Promise.resolve(interaction);
        });
    };
    return AudioTransferImplementation;
}());
exports.AudioTransferImplementation = AudioTransferImplementation;
