"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LUISServiceImplementation = void 0;
var log4js = require("log4js");
var config_1 = require("../config/config");
var promises_shared_1 = require("../shared/promises.shared");
var request_shared_1 = require("../shared/request.shared");
var logger = log4js.getLogger('Task');
var LUISServiceImplementation = (function () {
    function LUISServiceImplementation() {
    }
    LUISServiceImplementation.prototype.GetCognitiveEngineName = function () {
        return 'LUIS';
    };
    LUISServiceImplementation.prototype.GetCognitiveEngineCharsPerPhrase = function () {
        return config_1.COGNITIVE_SERVICES_LUIS_CHARS_PER_PHRASE;
    };
    LUISServiceImplementation.prototype.LoadAzureAccounts = function (subscriptionRegion, subscriptionKey, data) {
        var getOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL_ACCOUNTS,
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Ocp-Apim-Subscription-Key': subscriptionKey,
                'Authorization': data.token_type + ' ' + data.access_token
            }
        };
        logger.info('[LUISServiceImplementation::LoadAzureAccounts] getting data: ' + JSON.stringify(data));
        return request_shared_1.SimpleGetRequest(getOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::LoadAzureAccounts] getting response: ' + JSON.stringify(response));
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::LoadAzureAccounts] getting error:', errorMessage);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.ModelQuery = function (subscriptionRegion, subscriptionKey, appId, query, processAudios, nluName, resolve) {
        var getOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL_STUDY + "/" + appId + "?subscription-key=" + subscriptionKey + "&verbose=true&timezoneOffset=-300&q=" + encodeURIComponent(query),
        };
        logger.info('[LUISServiceImplementation::ModelQuery] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'query:', query, 'processAudios:', processAudios, 'nluName:', nluName);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then(function (response) {
            return Promise.resolve(response);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::ModelQuery] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'query:', query, 'processAudios:', processAudios, 'nluName:', nluName);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.DeleteAppIntent = function (subscriptionRegion, subscriptionKey, appId, intentId) {
        var deleteOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/intents/" + intentId + "/?deleteUtterances=false",
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::DeleteAppIntent] getting deleteOptions:', deleteOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'intentId:', intentId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::DeleteAppIntent] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'intentId:', intentId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::DeleteAppIntent] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'intentId:', intentId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.DeleteAppUtterance = function (subscriptionRegion, subscriptionKey, appId, intentId, utteranceId) {
        var deleteOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/examples/" + utteranceId,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::DeleteAppUtterance] getting deleteOptions:', deleteOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', utteranceId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::DeleteAppUtterance] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', utteranceId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::DeleteAppUtterance] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', utteranceId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.TrainApp = function (subscriptionRegion, subscriptionKey, appId) {
        var data = {};
        var postOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/train",
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::TrainApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimplePostRequest(postOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::TrainApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::TrainApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetTrainApp = function (subscriptionRegion, subscriptionKey, appId) {
        var data = {};
        var getOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/train",
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImp<lementation::GetTrainApp] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetTrainApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetTrainApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.PublishApp = function (subscriptionRegion, subscriptionKey, appId, isStaging) {
        var data = { versionId: '0.1', isStaging: isStaging, directVersionPublish: false };
        var postOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/publish",
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::PublishApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimplePostRequest(postOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::PublishApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::PublishApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetManagerToken = function (info) {
        var data = "grant_type=client_credentials"
            + "&client_id=" + info.client_id
            + "&client_secret=" + encodeURIComponent(info.client_secret)
            + "&resource=" + encodeURIComponent(info.resource);
        var postOptions = {
            uri: 'https://login.microsoftonline.com/' + info.tenant_id + '/oauth2/token',
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded'
            },
            body: data
        };
        logger.info('[LUISServiceImplementation::GetManagerToken] getting data: ' + JSON.stringify(info));
        return request_shared_1.SimplePostRequest(postOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetManagerToken] getting response: ' + JSON.stringify(response));
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetManagerToken] getting error:', errorMessage);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.UnAssignSubscriptionKeyOfVCC = function (subscriptionRegion, subscriptionKey, appId, data) {
        logger.info('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting subscriptionRegion: ' + subscriptionRegion);
        logger.info('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting appId: ' + appId);
        var body = "azureSubscriptionId=" + data.azureSubscriptionId
            + "&resourceGroup=" + data.resourceGroup
            + "&accountName=" + data.accountName;
        var deleteOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/azureaccounts",
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Ocp-Apim-Subscription-Key': subscriptionKey,
                'Authorization': data.token_type + ' ' + data.access_token
            },
            body: body
        };
        logger.info('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting data: ' + JSON.stringify(data));
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting response: ' + JSON.stringify(response));
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting error:', errorMessage);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.AssignSubscriptionKeyOfVCC = function (subscriptionRegion, subscriptionKey, appId, data) {
        logger.info('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting subscriptionRegion: ' + subscriptionRegion);
        logger.info('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting appId: ' + appId);
        var body = "azureSubscriptionId=" + data.azureSubscriptionId
            + "&resourceGroup=" + data.resourceGroup
            + "&accountName=" + data.accountName;
        var postOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/azureaccounts",
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Ocp-Apim-Subscription-Key': subscriptionKey,
                'Authorization': data.token_type + ' ' + data.access_token
            },
            body: body
        };
        logger.info('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting data: ' + JSON.stringify(data));
        return request_shared_1.SimplePostRequest(postOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting response: ' + JSON.stringify(response));
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting error:', errorMessage);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetCultures = function (subscriptionRegion, subscriptionKey) {
        var getOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/cultures",
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::GetCultures] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetCultures] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetCultures] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.AddApp = function (subscriptionRegion, subscriptionKey, nlu) {
        var data = { name: nlu.name, description: nlu.description, culture: nlu.culture };
        var postOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/",
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::AddApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
        return request_shared_1.SimplePostRequest(postOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::AddApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AddApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.DeleteApp = function (subscriptionRegion, subscriptionKey, nlu) {
        var deleteOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + nlu.appId + "?force=true",
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::DeleteApp] getting deleteOptions:', deleteOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', nlu.appId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::DeleteApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', nlu.appId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::DeleteApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', nlu.appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetAppsList = function (subscriptionRegion, subscriptionKey) {
        logger.info('[LUISServiceImplementation::GetAppsList] getting subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/", 0, 500)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetAppsList] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetAppsList] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.AddIntent = function (subscriptionRegion, subscriptionKey, appId, name) {
        var data = { name: name };
        var postOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/intents",
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::AddIntent] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'name:', name);
        return request_shared_1.SimplePostRequest(postOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::AddIntent] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'name:', name);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AddIntent] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'name:', name);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.AddUtterance = function (subscriptionRegion, subscriptionKey, appId, utterance) {
        var entityLabels = [];
        if (utterance && utterance.entityLabels && utterance.entityLabels.length > 0) {
            utterance.entityLabels.map(function (el) {
                entityLabels.push({
                    entityName: el.entityName,
                    startCharIndex: el.start,
                    endCharIndex: el.end
                });
            });
        }
        var data = {
            text: utterance.name,
            intentName: utterance.intent,
            entityLabels: entityLabels
        };
        var postOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/example",
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::AddUtterance] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', utterance);
        return request_shared_1.SimplePostRequest(postOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::AddUtterance] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', utterance);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AddUtterance] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', utterance);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetAppVersion = function (subscriptionRegion, subscriptionKey, appId) {
        var getOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/",
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::GetAppVersion] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetAppVersion] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetAppVersion] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetIntentsList = function (subscriptionRegion, subscriptionKey, appId) {
        logger.info('[LUISServiceImplementation::GetIntentsList] getting subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/intents/", 0, 500)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetIntentsList] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetIntentsList] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetIntentsListDemo = function (subscriptionRegion, subscriptionKey, appId, resolve, reject) {
        logger.info('[LUISServiceImplementation::GetIntentsListDemo] getting subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/intents/", 0, 500)
            .then(function (response) {
            return resolve ? resolve(response) : Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetIntentsListDemo] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(false);
        });
    };
    LUISServiceImplementation.prototype.GetUtterancesList = function (subscriptionRegion, subscriptionKey, appId) {
        logger.info('[LUISServiceImplementation::GetUtterancesList] subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/examples/", 0, 500)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetUtterancesList] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetUtterancesList] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetUtterancesFromMD = function (appId) {
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return Promise.resolve([]);
        });
    };
    LUISServiceImplementation.prototype.AddSimpleEntity = function (subscriptionRegion, subscriptionKey, appId, entity) {
        var data = { name: entity.name };
        var postOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/entities",
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::AddEntitySimple] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entity);
        return request_shared_1.SimplePostRequest(postOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::AddEntitySimple] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entity);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AddEntitySimple] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entity);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.RenameEntity = function (subscriptionRegion, subscriptionKey, appId, entityId, entityName) {
        var data = { name: entityName };
        var putOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/entities/" + entityId,
            method: 'PUT',
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
        };
        logger.info('[LUISServiceImplementation::RenameEntity] getting postOptions:', putOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entityId);
        return request_shared_1.SimplePutRequest(putOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::RenameEntity] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entityId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::RenameEntity] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entityId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetEntitiesList = function (subscriptionRegion, subscriptionKey, appId) {
        var getOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/entities",
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::GetEntities] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetEntities] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetEntities] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetEntity = function (subscriptionRegion, subscriptionKey, appId, entityId) {
        var getOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/entities/" + entityId,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::GetEntity] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::GetEntity] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(response);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetEntity] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.DeleteAppEntity = function (subscriptionRegion, subscriptionKey, appId, entityId) {
        var deleteOptions = {
            uri: "https://" + subscriptionRegion + "." + config_1.LUIS_URL + "/" + appId + "/versions/0.1/entities/" + entityId,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        logger.info('[LUISServiceImplementation::DeleteEntity] getting deleteOptions:', deleteOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', entityId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then(function (response) {
            logger.info('[LUISServiceImplementation::DeleteEntity] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', entityId);
            return response;
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::DeleteEntity] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', entityId);
            return Promise.reject(err);
        });
    };
    LUISServiceImplementation.prototype.GetAllIntentsFromRASAContent = function (content) {
        return null;
    };
    LUISServiceImplementation.prototype.GetAllUtterancesFromRASAContent = function (content) {
        return null;
    };
    LUISServiceImplementation.prototype.ExportModel = function (nlu, res) {
        return null;
    };
    LUISServiceImplementation.prototype.GetResultsFromPaginatedRequest = function (subscriptionRegion, subscriptionKey, url, skip, take) {
        var _this = this;
        skip = skip ? skip : 0;
        take = take ? take : 500;
        var allResults = [];
        var getOptions = {
            uri: url + "?skip=" + skip + "&take=" + take,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            }
        };
        return request_shared_1.SimpleGetRequest(getOptions)
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_GET_LUIS_ITEMS_SERVICE_FAILED';
            if (result.length && result.length >= take) {
                allResults = result;
                return _this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, url, skip + take, take);
            }
            else {
                return Promise.resolve(result);
            }
        })
            .then(function (result) {
            allResults = allResults.concat(result);
            return Promise.resolve(allResults);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetResultsFromPaginatedRequest] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    };
    return LUISServiceImplementation;
}());
exports.LUISServiceImplementation = LUISServiceImplementation;
