"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetQueryIdathaProcessV2 = exports.SemanticQueryImplementation = void 0;
var log4js = require("log4js");
var moment = require("moment");
var config_1 = require("../config/config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var config_2 = require("../config/config");
var file_manager_1 = require("../shared/file-manager");
var billing_implementation_1 = require("../implementation/billing.implementation");
var idatha_service_1 = require("../services/idatha.service");
var request_shared_1 = require("../shared/request.shared");
var billing_batch_model_1 = require("../models/billing-batch.model");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'SEMANTIC_ANALYSIS';
var CURRENT_IMPLEMENTATION = 'SemanticQueryImplementation';
var SemanticQueryImplementation = (function () {
    function SemanticQueryImplementation() {
        this.errorData = new billing_batch_model_1.BatchError();
    }
    SemanticQueryImplementation.prototype.Execute = function (executionCode, rateRecursive, availableBatch, cancelledBatch) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_SEMANTIC_QUERY;
        var alowedRatio = 0;
        var notAlowedRatio = 0;
        var executeResult = true;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return redis_service_1.Redis.LRange('SemanticQuery', 0, ratio - 1);
        })
            .then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            result.map(function (r) {
                var arrQueue = r.split(':');
                if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId === arrQueue[4]; })) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(_this.SemanticQueryExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > notAlowedRatio) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('SemanticQuery', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[Execute] getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    SemanticQueryImplementation.prototype.SemanticQueryExecute = function (executionCode, availableBatch, cancelledBatch) {
        var _this = this;
        var currentQueueId = '';
        var query = {};
        var process = {};
        var queryId = '';
        var genericQueryId = '';
        var type = '';
        var typeId = '';
        var templateId = '';
        var currentUserAccessToken = {};
        var isError = false;
        var isComplete = false;
        var isCancel = false;
        var queueProgressStorage = '';
        var retry;
        var idatha = {
            ip: config_1.IDATHA_IP,
            httpPort: config_1.IDATHA_HTTP_PORT
        };
        var queryFilePath = '';
        return redis_service_1.Redis.RemoveQueueL('SemanticQuery')
            .then(function (result) {
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Creating queues');
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Current queue: ', result);
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Alowed Batches: ', availableBatch);
            var arrQueue = result.split(':');
            var promiseArray = [];
            if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId === arrQueue[4]; })) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticQueryExecute', result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticQuery', result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[4], 'SEMANTIC_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            var queueId = result[2];
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item_1 = queueId.split(':');
                if (item_1 && item_1.length) {
                    if (cancelledBatch.find(function (b) { return b.VCC === item_1[1] && b.BatchTypeId === item_1[4]; })) {
                        logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' REMOVING CANCELLED BATCH');
                        throw 'REMOVED_BY_CANCELLED';
                    }
                    genericQueryId = item_1[2];
                    type = item_1[3] ? item_1[3] : null;
                    typeId = item_1[4] ? item_1[4] : null;
                    if (type && typeId)
                        queueProgressStorage = item_1[0] + ':' + item_1[1] + ':' + item_1[3] + ':' + item_1[4];
                    if (type && !typeId)
                        queueProgressStorage = item_1[0] + ':' + item_1[1] + ':' + item_1[3];
                    logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', queueProgressStorage);
                    return mssql_service_1.MSSql.GetProcessTask(item_1[0], item_1[1]);
                }
                else {
                    logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (result) {
            process = result;
            if (!process)
                throw 'PROCESS_NOT_FOUND';
            return redis_service_1.Redis.GetRetry(currentQueueId);
        }).then(function (result) {
            retry = result && result.value ? result.value : 0;
            return mssql_service_1.MSSql.GetGenericQuery(process.vcc, genericQueryId);
        }).then(function (result) {
            query = result;
            if (!query || (query && !query.query))
                throw 'QUERY_IS_NOT_FOUND';
            var key = process.vcc + ':' + process.vcc;
            return idatha_service_1.IdathaLogin(idatha, process.vcc);
        }).then(function (userAccessToken) {
            if (!userAccessToken)
                throw 'NOT_FOUND_ACCESS_TOKEN';
            currentUserAccessToken = userAccessToken;
            return _this.GetIdathaQueryId(idatha, currentUserAccessToken.accessToken, process, executionCode, currentQueueId);
        }).then(function (resultQueryId) {
            queryId = resultQueryId;
            if (!queryId)
                throw 'NOT_FOUND_QUERY_ID';
            if (type === 'study' && typeId)
                return mssql_service_1.MSSql.GetStudy(process.vcc, typeId);
            return null;
        }).then(function (result) {
            if (result && type === 'study')
                templateId = result.templateId;
            queryFilePath = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
            return file_manager_1.DeleteFile(queryFilePath);
        }).then(function (result) {
            return _this.GetIdathaQuery(idatha, currentUserAccessToken.accessToken, process.vcc, queryId, query.name, query.query, 0, 15, executionCode, currentQueueId, query.version, process);
        }).then(function (result) {
            if (result === null)
                throw 'IDATHA_REQUEST_FAILED';
            if (typeof result === 'boolean' && result === false)
                throw 'NOT_FOUND_DATA';
            return _this.GetDocumentQuery(executionCode, currentQueueId);
        }).then(function (results) {
            if (type === 'study' && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_STUDY');
                return mssql_service_1.MSSql.BulkInteractionQueryStudyTemp(process.vcc, process.name, typeId, templateId, genericQueryId, results[query.name]);
            }
            else if (type === 'evaluationPlan' && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_EVALUATION_PLAN_CONTENT_SEARCH');
                return mssql_service_1.MSSql.BulkEvaluationPlanQueryTemp(process.vcc, process.name, typeId, genericQueryId, results[query.name]);
            }
            else if (type === 'ability' && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_ABILITY');
                return mssql_service_1.MSSql.BulkEvaluationPlanAbilityTemp(process.vcc, process.name, typeId, genericQueryId, results[query.name]);
            }
            else if (process && type === 'search') {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_AUDIO');
                return mssql_service_1.MSSql.BulkInteractionQueryTemp(process.vcc, process.name, genericQueryId, results[query.name]);
            }
            else if (results && results[query.name]) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_NONE');
                return true;
            }
            return null;
        }).then(function (result) {
            isComplete = result ? true : false;
            if (!isComplete) {
                retry++;
                return redis_service_1.Redis.UpdateRetry(currentQueueId);
            }
            return isComplete;
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            if (!_this.errorData.errorFunction)
                _this.errorData.errorFunction = 'SemanticQueryExecute';
            if (!_this.errorData.errorFunction)
                _this.errorData.errorMessage = errorMessage ? errorMessage : 'ERROR';
            if (!_this.errorData.errorDetail)
                _this.errorData.errorFunction = err;
            if (errorMessage === 'PROCESS_NOT_FOUND'
                || errorMessage === 'QUERY_IS_NOT_FOUND'
                || errorMessage === 'NOT_FOUND_ACCESS_TOKEN'
                || errorMessage === 'NOT_FOUND_QUERY_ID'
                || errorMessage === 'NOT_FOUND_DATA'
                || errorMessage === 'PROCESS_FILTER_NOT_FOUND'
                || errorMessage === 'REMOVED_BY_CANCELLED') {
                isError = true;
            }
            else {
                isComplete = false;
                retry++;
                return redis_service_1.Redis.UpdateRetry(currentQueueId);
            }
            return false;
        }).then(function (result) {
            if (isComplete) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                _this.CloseSemanticQuery(currentQueueId, true, 'SUCCED');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else if (isError) {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_ERRORS_QUEUE_FINISHED');
                _this.CloseSemanticQuery(currentQueueId, false, 'REMOVED_BY_ERRORS_QUEUE_FINISHED');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else if (config_1.RETRY_SEMANTIC_QUERY <= retry) {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRY');
                _this.CloseSemanticQuery(currentQueueId, false, 'REMOVED_BY_RETRY');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return null;
        }).then(function (result) {
            var promiseArray = [];
            if (isComplete) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteRetry(currentQueueId));
            }
            else if (isError
                ||
                    config_1.RETRY_SEMANTIC_QUERY <= retry) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteRetry(currentQueueId));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(process.vcc, CURRENT_SYSTEM_SERVICE, typeId, typeId === 'study' ? typeId : null, CURRENT_IMPLEMENTATION, null, query.name, _this.errorData.errorFunction, _this.errorData.errorMessage, _this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    SemanticQueryImplementation.prototype.GetIdathaQueryId = function (idatha, accessToken, process, executionCode, currentQueueId) {
        var _this = this;
        var queryId = '';
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            var currentDate = new Date();
            var queryDataIdDate = new Date(process.queryDataIdDate);
            var diffDays = Math.round(Math.abs((currentDate.getTime() - queryDataIdDate.getTime()) / (24 * 60 * 60 * 1000)));
            if (!process.queryDataId || diffDays >= config_1.IDATHA_QUERY_ID_EXPIRATION) {
                if (!process.processFilters)
                    throw 'PROCESS_FILTER_NOT_FOUND';
                var queryProcess = GetQueryIdathaProcessV2(process.processFilters);
                var options = _this.GetIdathaRequestOptions(process.vcc, idatha, accessToken, queryId, queryProcess, 0, 1, 2, process.processFilters);
                logger.info('[SemanticQueryImplementation::GetIdathaQueryId] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'option request:', options);
                return request_shared_1.SimplePostRequest(options.body);
            }
            queryId = process.queryDataId;
            return null;
        }).then(function (result) {
            if (result) {
                queryId = result.query_id;
                return mssql_service_1.MSSql.SaveProcess(process.vcc, process.name, process.description, process.clientId, process.status, process.type, process.startDate, process.endDate, process.minDurationTime, process.maxDurationTime, process.language, process.dictionaryId, process.environment, process.queryId, queryId, moment().utc().toDate(), process.createdDate, process.createdByUserId, process.lastModifiedDate, process.lastModifiedByUserId);
            }
            return null;
        }).then(function (result) {
            _this.errorData.CleanErrorData();
            return Promise.resolve(queryId);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::GetIdathaQueryId] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('GetIdathaQueryId', errorMessage, err);
            return Promise.resolve(null);
        });
    };
    SemanticQueryImplementation.prototype.GetIdathaQueryType = function (queryString) {
        var queryObject = null;
        var queryType = 'REGULAR';
        try {
            queryObject = JSON.parse(queryString);
            if (typeof queryObject === 'object') {
                if (queryObject.filter.includes('silences:'))
                    queryType = 'SILENCES';
            }
        }
        catch (e) {
            logger.info('[SemanticQueryImplementation::GetIdathaQueryType] Could not parse query:', queryString);
        }
        return {
            queryType: queryType,
            queryString: queryString,
            queryObject: queryObject
        };
    };
    SemanticQueryImplementation.prototype.GetIdathaRequestOptions = function (vcc, idatha, accessToken, queryId, queryString, from, size, version, processFilters) {
        if (version === void 0) { version = 1; }
        if (!idatha) {
            idatha = {
                ip: config_1.IDATHA_IP,
                httpPort: config_1.IDATHA_HTTP_PORT
            };
        }
        var uri = null;
        var path = null;
        var body = {};
        var bodyParsed = null;
        var idathaQueryType = this.GetIdathaQueryType(queryString);
        var queryType = idathaQueryType.queryType;
        var queryObject = idathaQueryType.queryObject;
        switch (queryType) {
            case 'SILENCES':
                path = vcc + "_entries/_search";
                idatha.httpPort = '9200';
                var must = [];
                var silencesRange = { boost: 1.0 };
                var speaker = null;
                var startDate = processFilters.find(function (pf) { return pf.key === 'startDate'; }).value;
                var endDate = processFilters.find(function (pf) { return pf.key === 'endDate'; }).value;
                startDate = startDate.slice(0, 10);
                endDate = endDate.slice(0, 10);
                if (queryObject.minType.length) {
                    silencesRange[queryObject.minType.toLowerCase()] = queryObject.minValue;
                    silencesRange['include_lower'] = queryObject.minType.includes('e') ? true : false;
                }
                if (queryObject.maxType.length) {
                    silencesRange[queryObject.maxType.toLowerCase()] = queryObject.maxValue;
                    silencesRange['include_upper'] = queryObject.maxType.includes('e') ? true : false;
                }
                must.push({ range: { "words.silence": silencesRange } });
                if (queryObject.scope && queryObject.scope !== 'TEXT_ALL') {
                    if (queryObject.scope === 'in_sentences.')
                        speaker = 'in';
                    if (queryObject.scope === 'out_sentences.')
                        speaker = 'out';
                    if (speaker)
                        must.push({ match: { "words.speaker": speaker } });
                }
                bodyParsed = {
                    from: from,
                    size: size,
                    query: {
                        bool: {
                            filter: [
                                {
                                    bool: {
                                        must: [
                                            {
                                                nested: {
                                                    query: {
                                                        bool: {
                                                            must: must
                                                        }
                                                    },
                                                    path: "words",
                                                    ignore_unmapped: false,
                                                    score_mode: "none",
                                                    boost: 1.0,
                                                    inner_hits: {
                                                        ignore_unmapped: false,
                                                        from: 0,
                                                        size: 100,
                                                        version: false,
                                                        explain: false,
                                                        track_scores: false,
                                                        _source: {
                                                            includes: ["words.silence", "words.name", "words.speaker", "words.time"],
                                                            excludes: []
                                                        }
                                                    }
                                                }
                                            },
                                            {
                                                nested: {
                                                    query: {
                                                        bool: {
                                                            must: [
                                                                {
                                                                    range: {
                                                                        "filter_data.end_date": {
                                                                            gte: startDate + "T00:00:00",
                                                                            lte: endDate + "T23:59:59",
                                                                            include_lower: true,
                                                                            include_upper: true,
                                                                            boost: 1.0
                                                                        }
                                                                    }
                                                                }
                                                            ]
                                                        }
                                                    },
                                                    path: "filter_data",
                                                    ignore_unmapped: false,
                                                    score_mode: "none",
                                                    boost: 1.0,
                                                    inner_hits: {
                                                        ignore_unmapped: false,
                                                        from: 0,
                                                        size: 100,
                                                        version: false,
                                                        explain: false,
                                                        track_scores: false,
                                                        _source: {
                                                            includes: ["words.silence", "words.name", "words.speaker", "words.time"],
                                                            excludes: []
                                                        }
                                                    }
                                                }
                                            }
                                        ],
                                        adjust_pure_negative: true,
                                        boost: 1.0
                                    }
                                }
                            ],
                            adjust_pure_negative: true,
                            boost: 1.0
                        }
                    }
                };
                break;
            default:
                path = 'api/query';
                if (version === 2) {
                    var dataObject = {
                        version: version,
                        from: from,
                        size: size,
                        query: queryString
                    };
                    if (queryId)
                        dataObject.query_id = queryId;
                    var data = JSON.stringify(dataObject);
                    bodyParsed = JSON.parse(data.replace(/\\\\\\/g, "\\"));
                }
                else {
                    var stringQuery = '{"query_id": "' + queryId + '", "query": "' + queryString.replace(/\"/g, "\\\\\\\\\\\\\"") + '", "from": ' + from + ', "size": ' + size + ',"version":' + (version ? version : 1) + '}';
                    bodyParsed = JSON.parse(stringQuery);
                }
                break;
        }
        uri = "http://" + idatha.ip + ":" + idatha.httpPort + "/";
        uri += path;
        body = {
            uri: uri,
            body: bodyParsed,
            headers: {
                'Authorization': 'Bearer ' + accessToken,
                'Content-Type': 'application/json'
            }
        };
        return {
            queryType: queryType,
            body: body
        };
    };
    SemanticQueryImplementation.prototype.GetIdathaRequestResponse = function (queryType, queryString, rawResponse) {
        var response = {};
        switch (queryType) {
            case 'SILENCES':
                var queryObject_1 = JSON.parse(queryString);
                response.total_results = rawResponse.hits.total;
                response.results = [];
                rawResponse.hits.hits.map(function (h) {
                    h._source.matches = h.inner_hits.words.hits.total;
                    h._source.matchesDetail = [];
                    h.inner_hits.words.hits.hits.map(function (hh) { return h._source.matchesDetail.push(hh._source); });
                    if (h._source.matches >= queryObject_1.minOcurrencies)
                        response.results.push(h._source);
                });
                break;
            default:
                response = rawResponse;
                break;
        }
        return response;
    };
    SemanticQueryImplementation.prototype.GetIdathaQuery = function (idatha, accessToken, vcc, queryId, queryName, query, from, size, executionCode, currentQueueId, version, process) {
        var _this = this;
        if (version === void 0) { version = 1; }
        var isNext = true;
        var queryType = null;
        var queryResultTotal = 0;
        var queryResultPages = 0;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            var options = _this.GetIdathaRequestOptions(vcc, idatha, accessToken, queryId, query, from, size, version, process.processFilters);
            queryType = options.queryType;
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'option request:', options);
            return request_shared_1.SimplePostRequest(options.body);
        })
            .then(function (result) {
            var response = _this.GetIdathaRequestResponse(queryType, query, result);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Items per page: ' + size);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Total results: ' + response.total_results);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Results for this request: ' + response.results.length);
            isNext = from + size <= (response.total_results * 1) ? true : false;
            queryResultTotal = response.total_results;
            queryResultPages = response.total_results > 0 ? Math.ceil(response.total_results / size) : 1;
            var pattern = /\in_sentences|\out_sentences/g;
            if (query.match(pattern)) {
                var orPattern1 = /((?!out_sentences|in_sentences)\w+\s(OR)\s.*(in_sentences|out_sentences)[^\s]+)/g;
                var orPattern2 = /((out_sentences|in_sentences)[^\s]+\s(OR)\s((?!out_sentences|in_sentences)\w+[^\s]))/g;
                if (query.match(orPattern1) || query.match(orPattern2)) {
                }
                else {
                    response.results = response.results.filter(function (r) { return r && r.filter_data && r.filter_data.channel && r.filter_data.channel !== 'MAIL'; });
                }
            }
            return _this.SaveDocumentQuery(response.results, queryName, executionCode, currentQueueId);
        }).then(function (result) {
            if (!result)
                throw 'ERROR_GET_IDATHA_INTERACTIONS';
            if (isNext)
                return _this.GetIdathaQuery(idatha, accessToken, vcc, queryId, queryName, query, (from + size), size, executionCode, currentQueueId, version, process);
            return true;
        }).then(function (result) {
            _this.SaveSemanticQueryResult(currentQueueId, queryResultTotal, queryResultPages, size);
            _this.errorData.CleanErrorData();
            return Promise.resolve(result);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.debug('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting rawError:', err);
            logger.error('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('GetIdathaRegularQuery', errorMessage, err);
            if (err && err.statusCode && err.statusCode === 500) {
                return Promise.resolve(null);
            }
            else {
                _this.CloseSemanticQuery(currentQueueId, false, '');
                return Promise.resolve(false);
            }
        });
    };
    SemanticQueryImplementation.prototype.SaveDocumentQuery = function (data, id, executionCode, currentQueueId) {
        var _this = this;
        var fileName = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
        return promises_shared_1.DummyPromise()
            .then(function () {
            return file_manager_1.CreateFolder(config_2.QUERY_FOLDER);
        }).then(function () {
            return file_manager_1.ReadFileContent(fileName, 'utf-8', true);
        }).then(function (resultFile) {
            var jsonContent;
            var results = data ? data : [];
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                    jsonContent[id] = jsonContent[id] ? jsonContent[id].concat(results) : [];
                }
                else {
                    jsonContent = {};
                    jsonContent[id] = results;
                }
            }
            catch (ex) {
                logger.error('[SemanticQueryImplementation::SaveDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                throw 'ERROR_FILE_JSON';
            }
            return file_manager_1.WriteFileContent(fileName, JSON.stringify(jsonContent), 'utf-8', true);
        }).then(function (result) {
            _this.errorData.CleanErrorData();
            return Promise.resolve(true);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticQueryImplementation::SaveDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('SaveDocumentQuery', errorMessage, error);
            return Promise.resolve(false);
        });
    };
    SemanticQueryImplementation.prototype.GetDocumentQuery = function (executionCode, currentQueueId) {
        var _this = this;
        var fileName = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
        return promises_shared_1.DummyPromise()
            .then(function () {
            return file_manager_1.ReadFileContent(fileName, 'utf-8', true);
        }).then(function (resultFile) {
            var jsonContent;
            try {
                jsonContent = JSON.parse(resultFile);
            }
            catch (ex) {
                jsonContent = null;
                throw 'ERROR_FILE_JSON';
            }
            _this.errorData.CleanErrorData();
            return Promise.resolve(jsonContent);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticQueryImplementation::GetDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('GetDocumentQuery', errorMessage, error);
            return Promise.resolve(null);
        });
    };
    SemanticQueryImplementation.prototype.SaveSemanticQueryResult = function (queueId, totalResults, totalPages, totalItemsPerPage) {
        var billing = new billing_implementation_1.BillingImplementation();
        var myBatchName = '';
        var myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[3]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'evaluationPlan':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'search':
                myBatchName = '';
                break;
        }
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalResults', totalResults);
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalPages', totalPages);
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalItemsPerPage', totalItemsPerPage);
    };
    SemanticQueryImplementation.prototype.CloseSemanticQuery = function (queueId, resultCode, resultDescription) {
        var billing = new billing_implementation_1.BillingImplementation();
        var myBatchName = '';
        var myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[3]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'evaluationPlan':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'search':
                myBatchName = '';
                break;
        }
        billing.AddCloseSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], resultCode ? 1 : 0, resultDescription);
    };
    return SemanticQueryImplementation;
}());
exports.SemanticQueryImplementation = SemanticQueryImplementation;
function GetQueryIdathaProcessV2(processFilters) {
    var queries = [];
    var campaigns = [];
    var agents = [];
    var channels = [];
    var startDate = '';
    var endDate = '';
    for (var _i = 0, processFilters_1 = processFilters; _i < processFilters_1.length; _i++) {
        var p = processFilters_1[_i];
        if (p.value) {
            if (p.key == 'campaigns') {
                campaigns.push("filter_data:(campaign:\"" + p.value + "\")");
            }
            if (p.key == 'agents') {
                agents.push("filter_data:(agent:\"" + p.value + "\")");
            }
            if (p.key == 'endDate') {
                endDate = p.value;
            }
            if (p.key == 'startDate') {
                startDate = p.value;
            }
            if (p.key == 'types') {
                channels.push("filter_data:(channel:\"" + p.value + "\")");
            }
        }
    }
    if (startDate != '' && endDate != '') {
        startDate = startDate.slice(0, 10);
        endDate = endDate.slice(0, 10);
        queries.push("filter_data:(end_date:[" + startDate + " TO " + endDate + "])");
    }
    if (agents.length > 0) {
        queries.push("(" + agents.join(' OR ') + ")");
    }
    if (campaigns.length > 0) {
        queries.push("(" + campaigns.join(' OR ') + ")");
    }
    if (channels.length > 0) {
        queries.push("(" + channels.join(' OR ') + ")");
    }
    return queries.join(' AND ');
}
exports.GetQueryIdathaProcessV2 = GetQueryIdathaProcessV2;
