"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAgents = exports.AllegroService = void 0;
var log4js = require("log4js");
var redis_service_1 = require("../db/redis.service");
var interaction_model_1 = require("../models/interaction.model");
var mssql_config_1 = require("../config/mssql.config");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var integration_config_1 = require("../config/integration.config");
var http_request_occ_1 = require("../shared/http-request-occ");
var interaction_service_1 = require("./interaction.service");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var logger = log4js.getLogger('ServerScripts');
function AllegroService(app) {
    app.get('/api/process/allegro/get_allegro_campaigns', function (req, res, next) {
        logger.info('[AllegroService::get_allegro_campaigns] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var type = res.locals.type;
        var campaigns = [];
        var allegroCampaign = [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaigns(integration_config_1.IP_BD, vcc, flagAllegro));
                    break;
                case 'I6':
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    break;
                case 'OCC':
                    var flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaigns(integration_config_1.IP_BD, vcc, flagOCC));
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    break;
                case 'OTHER':
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            switch (type) {
                case 'ALLEGRO':
                    if (!result[0])
                        throw 'SERVER_ERROR_GET_SQL_CAMPAIGNS_FAILED';
                    campaigns = result[0];
                    break;
                case 'I6':
                    if ((!result[0]) || (result[0] && result[0].status === false))
                        throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
                    campaigns = result[0].body;
                    break;
                case 'OCC':
                    if (!result[0]
                        &&
                            ((!result[1]) || (result[1] && result[1].status === false)))
                        throw 'SERVER_ERROR_GET_CAMPAIGNS_FAILED';
                    var allegroCampaigns = result[0] ? result[0] : [];
                    var campaignsApi_1;
                    if (result[1] && result[1].status === true) {
                        campaignsApi_1 = result[1] && result[1].status ? result[1].body : [];
                        campaigns = allegroCampaigns.filter(function (c) { return campaignsApi_1.find(function (ca) { return ca.id === c.id; }); });
                    }
                    else {
                        campaigns = allegroCampaigns;
                    }
                    break;
                case 'OTHER':
                    break;
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: campaigns });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AllegroService::get_allegro_campaigns] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/process/allegro/get_allegro_agents/:campaign?', function (req, res, next) {
        logger.info('[AllegroService::get_allegro_agents] Receives ' + req.method + ' method on \'' + req.path + '\'');
        var campaign = req.params.campaign || '';
        var vcc = res.locals.vcc;
        var type = res.locals.type;
        var agents = [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return GetAgents(vcc, type, campaign);
        })
            .then(function (result) {
            agents = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: agents });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AllegroService::get_allegro_agents] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/process/allegro/get_allegro_management_results', function (req, res, next) {
        logger.info('[AllegroService::get_allegro_management_results] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var type = res.locals.type;
        var allegroManagementResult = [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroManagementResults(integration_config_1.IP_BD, vcc, flagAllegro));
                    break;
                case 'I6':
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    break;
                case 'OCC':
                    var flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroManagementResults(integration_config_1.IP_BD, vcc, flagOCC));
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    break;
                case 'OTHER':
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            switch (type) {
                case 'ALLEGRO':
                    if (!result[0])
                        throw 'SERVER_ERROR_GET_SQL_DISPOSITION_CODES_FAILED';
                    allegroManagementResult = result[0];
                    break;
                case 'I6':
                    if ((!result[0]) || (result[0] && result[0].status === false))
                        throw 'SERVER_ERROR_GET_API_I6_DISPOSITION_CODES_FAILED';
                    var campaignsApiI6 = result[0].body;
                    campaignsApiI6.map(function (c) {
                        if (c.dispositionCodes) {
                            c.dispositionCodes.map(function (dc) {
                                var dispositionCode = {};
                                dispositionCode.campaignId = c.id;
                                dispositionCode.description = dc.parent ? dc.parent : null;
                                dispositionCode.description2 = dc.child ? dc.child : null;
                                dispositionCode.description3 = dc.child2 ? dc.child2 : null;
                                dispositionCode.description4 = dc.child3 ? dc.child3 : null;
                                allegroManagementResult.push(dispositionCode);
                            });
                        }
                    });
                    break;
                case 'OCC':
                    if (!result[0]
                        &&
                            ((!result[1]) || (result[1] && result[1].status === false)))
                        throw 'SERVER_ERROR_GET_DISPOSITION_CODES_FAILED';
                    allegroManagementResult = result[0];
                    var campaignsApi = result[1] && result[1].status ? result[1].body : [];
                    campaignsApi.map(function (c) {
                        if (c.dispositionCodes) {
                            c.dispositionCodes.map(function (dc) {
                                var dispositionCode = {};
                                dispositionCode.vcc = vcc;
                                dispositionCode.code = null;
                                dispositionCode.campaignId = c.id;
                                dispositionCode.description = dc.parent ? dc.parent : null;
                                dispositionCode.description2 = dc.child ? dc.child : null;
                                dispositionCode.description3 = dc.child2 ? dc.child2 : null;
                                dispositionCode.description4 = dc.child3 ? dc.child3 : null;
                                allegroManagementResult.push(dispositionCode);
                            });
                        }
                    });
                    break;
                case 'OTHER':
                    break;
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: allegroManagementResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/allegro/get_allegro_summary_interactions', function (req, res, next) {
        logger.info('[AllegroService::get_allegro_summary_interactions] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var params = new interaction_model_1.InteractionSearch;
        params = interaction_service_1.GetInteractionSearchFromRequest(req.body.evaluationFormatId, req.body, false);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (params.environmentType && (params.environmentType === 'allegro' || params.environmentType === 'i6')) {
                return Promise.resolve(true);
            }
            else {
                var ftpCSVRedisKey = void 0;
                if (params.evaluationFormatId)
                    ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                else
                    ftpCSVRedisKey = 'FTP_PROCESS:';
                return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
            }
        })
            .then(function (result) {
            if (params.environmentType !== 'allegro' && params.environmentType !== 'i6')
                if (result === null)
                    throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            var promiseArray = [];
            promiseArray = interaction_service_1.GetChannelNewInteractionSummaryPromises(vcc, type, true, 'SUMMARIZED', params, result.data);
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var arrResult = [];
            arrResult = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: arrResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/interaction/allegro/get_allegro_billing_interactions', function (req, res, next) {
        logger.info('[AllegroService::get_billing_interactions] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var processId = req.body.processId || null;
        var modelSecondsPerRequest = req.body.secondsPerRequestRate || 0;
        var params = new interaction_model_1.InteractionSearch;
        var ftpCSVRedisKey;
        var cognitiveService;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (processId) {
                return Promise.resolve([true]);
            }
            else {
                params = interaction_service_1.GetInteractionSearchFromRequest(req.body.evaluationFormatId, req.body.filters, true);
                var promiseArray = [];
                if (params.evaluationFormatId && params.campaigns && params.campaigns.length === 0) {
                    logger.info('[AllegroService::get_billing_interactions] Searching evaluation format\'s campaigns');
                    ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                    promiseArray.push(mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId));
                }
                else {
                    ftpCSVRedisKey = 'FTP_PROCESS:';
                    promiseArray.push(Promise.resolve([true]));
                }
                if (params.environmentType && (params.environmentType !== 'allegro' && params.environmentType !== 'i6')) {
                    promiseArray.push(redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId));
                }
                else {
                    promiseArray.push(Promise.resolve([true]));
                }
                return Promise.all(promiseArray);
            }
        })
            .then(function (result) {
            var promise = null;
            if (processId) {
                promise = mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, processId, modelSecondsPerRequest);
            }
            else {
                logger.info('[AllegroService::get_billing_interactions] Requesting not evaluated interactions from HistoricalData');
                if (params.environmentType
                    &&
                        (params.environmentType === 'allegro'
                            ||
                                params.environmentType === 'i6')) {
                    if (params.campaigns && params.campaigns.length === 0) {
                        var evaluationFormatCampaigns = [];
                        evaluationFormatCampaigns = result[0].map(function (r) { return r.id; });
                        params.campaigns = evaluationFormatCampaigns;
                    }
                }
                else {
                    if (result[1] === null)
                        throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
                }
                promise = interaction_service_1.GetChannelOldInteractionSummaryPromises(vcc, type, modelSecondsPerRequest, cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 100, params, result[1].data ? result[1].data : []);
            }
            return promise;
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AllegroService = AllegroService;
function GetAgents(vcc, vccType, campaign) {
    var agents = [];
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var promiseArray = [];
        switch (vccType) {
            case 'ALLEGRO':
                var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                promiseArray.push(mssql_service_1.MSSql.GetAllegroAgents(integration_config_1.IP_BD, vcc, flagAllegro, campaign));
                break;
            case 'I6':
                promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, campaign));
                break;
            case 'OCC':
                var flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                promiseArray.push(mssql_service_1.MSSql.GetAllegroAgents(integration_config_1.IP_BD, vcc, flagOCC, campaign));
                promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, campaign));
                break;
            case 'OTHER':
                break;
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        switch (vccType) {
            case 'ALLEGRO':
                if (!result[0])
                    throw 'SERVER_ERROR_GET_SQL_AGENTS_FAILED';
                agents = result[0];
                break;
            case 'I6':
                if ((!result[0]) || (result[0] && result[0].status === false))
                    throw 'SERVER_ERROR_GET_API_I6_AGENTS_FAILED';
                agents = result[0].body;
                break;
            case 'OCC':
                if (!result[0]
                    &&
                        ((!result[1]) || (result[1] && result[1].status === false)))
                    throw 'SERVER_ERROR_GET_AGENTS_FAILED';
                var allegroAgents = result[0] ? result[0] : [];
                allegroAgents.map(function (aa) { return aa.userName = aa.userName.replace('  ', ' '); });
                var agentsApi = void 0;
                if (result[1] && result[1].status === true) {
                    agentsApi = result[1] && result[1].status ? result[1].body : [];
                    agents = allegroAgents.concat(agentsApi);
                }
                else {
                    agents = allegroAgents;
                }
                break;
            case 'OTHER':
                break;
        }
        return Promise.resolve(agents);
    })
        .catch(function (err) {
        logger.error('[AllegroService::GetAgents] Error al procesar: ', err);
        return Promise.reject(err);
    });
}
exports.GetAgents = GetAgents;
