"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAudioFileFromIntegrated = exports.GetAudioFileFromSftp = exports.GetAudioFileFromFtp = exports.IdathaDocumentWords = exports.GetIdathaDocument = exports.AudioService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var csvtojson = require('csvtojson');
var alasql = require('alasql');
var Client = require('ftp');
var SSH2Promise = require("ssh2-promise");
var mssql_service_1 = require("../db/mssql.service");
var file_manager_1 = require("../shared/file-manager");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var config_2 = require("../config/config");
var integration_config_1 = require("../config/integration.config");
var utilities_1 = require("../shared/utilities");
var idatha_service_1 = require("./idatha.service");
var vcc_service_1 = require("./vcc.service");
var diarization_analysis_implementation_1 = require("../implementation/diarization-analysis.implementation");
var mssql_config_1 = require("../config/mssql.config");
var request_shared_1 = require("../shared/request.shared");
var http_request_occ_1 = require("../shared/http-request-occ");
var logger = log4js.getLogger('ServerScripts');
function AudioService(app) {
    app.get('/api/process/audio/get_audio/:processId/:id?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.processId;
        var id = req.params.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (processId !== 'null') {
                var file_1 = config_2.PATH_FOLDER_PROCESS_AUDIO_PLAY + '\/' + processId + '\/' + id + '.mp3';
                return file_manager_1.BufferFile(file_1)
                    .then(function (result) {
                    return Promise.resolve(result);
                })
                    .catch(function (err) {
                    file_1 = config_2.PATH_FOLDER_PROCESS_AUDIO_PLAY + '\/' + processId + '\/' + id + '.wav';
                    return file_manager_1.BufferFile(file_1)
                        .then(function (result) {
                        return Promise.resolve(result);
                    });
                });
            }
            else {
                var url = id;
                url = Buffer.from(url, 'base64').toString();
                return ExternalRequest(url);
            }
        }).then(function (result) {
            res.json({ result: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_audio_navigator_interactions/:processId', function (req, res) {
        var vcc = res.locals.vcc;
        var processId = req.params.processId;
        promises_shared_1.DummyPromise().then(function (_) {
            return mssql_service_1.MSSql.GetAudioNavigatorInteractions(vcc, processId);
        })
            .then(function (interactions) {
            res.json({ status: true, data: interactions });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/audio/get_audios_process', function (req, res, next) {
        logger.info('[AudioService::get_audios_process] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var newAudio = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var times = getTimes(newAudio.startTime, newAudio.endTime);
            return mssql_service_1.MSSql.GetAudiosProcess(newAudio.pageSize, newAudio.pageNumber, vcc, newAudio.processId, newAudio.id, newAudio.startDate, newAudio.endDate, times.startTime.hour, times.startTime.minute, times.endTime.hour, times.endTime.minute, newAudio.campaign, newAudio.agent, newAudio.resultCode, newAudio.type, newAudio.idathaResults, newAudio.totalIdathaResults, newAudio.interactionType, (newAudio.interactionIds || []).toString(), newAudio.folder);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AudioService::get_audios_process] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/process/audio/get_audios_transcriptions', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var newAudio = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var times = getTimes(newAudio.startTime, newAudio.endTime);
            var interactionIds = (newAudio.interactionIds || []).toString();
            return mssql_service_1.MSSql.GetAudiosTranscriptions(newAudio.pageSize, newAudio.pageNumber, vcc, newAudio.processId, newAudio.id, newAudio.startDate, newAudio.endDate, times.startTime.hour, times.startTime.minute, times.endTime.hour, times.endTime.minute, newAudio.campaign, newAudio.agent, newAudio.resultCode, interactionIds);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/allegro/get_allegro_audios_statistic/:id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var id = req.params.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAllegroAudiosStatistic(vcc, id);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_audio_transcription/:process_id/:audio_id/:section/:part', function (req, res, next) {
        req.setTimeout(300000, function () { });
        logger.info('[AudioService::get_audio_transcription] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.process_id === 'null' ? null : req.params.process_id;
        var interactionType = req.params.interactionType;
        var audioId = req.params.audio_id;
        var audioData = {};
        var section = req.params.section * 1;
        var part = req.params.part * 1;
        var environment = null;
        var _resultReadFileContent = null;
        var _transcriptionQualification = 0;
        var buffer = null;
        var resultAudio;
        var interactionIdData;
        var fileNameAudio = '';
        var fileNameAnalysisSemantic = '';
        var fileNameAnalysisSemanticView = '';
        var jsonFileName = '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (processId === null) {
                interactionIdData = utilities_1.getInteractionIdFromURLBase64(audioId);
                logger.info('[AudioService::get_audio_transcription] Getting interactionIdData:', interactionIdData, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
                if (interactionIdData) {
                    audioId = interactionIdData.id ? interactionIdData.id : audioId;
                    section = section || section === 0 ? section : interactionIdData.section;
                    part = part || part === 0 ? part : interactionIdData.part;
                }
                else {
                    throw 'INVALID_AUDIO_DATA';
                }
            }
            return mssql_service_1.MSSql.GetAudio(vcc, processId, audioId, section, part);
        }).then(function (result) {
            resultAudio = result;
            if (resultAudio) {
                processId = processId ? processId : result.processId;
                audioData = {
                    audioId: audioId,
                    relativePath: resultAudio.relativePath,
                    fileName: resultAudio.fileName
                };
                return mssql_service_1.MSSql.GetProcess(vcc, processId);
            }
            else {
                return mssql_service_1.MSSql.GetEvaluationPlanByInteractionId(vcc, audioId);
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AUDIO_SYSTEM_SERVICE_NOT_FOUND';
            if (result.relativePath !== undefined && result.fileName !== undefined) {
                audioData = {
                    audioId: audioId,
                    relativePath: result.relativePath,
                    fileName: result.fileName
                };
            }
            return mssql_service_1.MSSql.GetEnvironment(vcc, (result.environment || result.environmentId));
        }).then(function (result) {
            if (!result)
                throw 'ENVIRONMENT_NOT_FOUND';
            environment = result;
            return GetAudioFileFromIntegrated(vcc, null, audioId, section, part, resultAudio && resultAudio.fileName ? resultAudio.fileName : null, null, 'LOCAL');
        }).catch(function (err) {
            if (typeof err === 'string'
                &&
                    [
                        'INVALID_AUDIO_DATA',
                        'SERVER_ERROR_AUDIO_SYSTEM_SERVICE_NOT_FOUND',
                        'ENVIRONMENT_NOT_FOUND'
                    ].includes(err))
                throw err;
            return null;
        }).then(function (result) {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                if (environment.type === 'allegro' || environment.type === 'i6') {
                    return GetAudioFileFromIntegrated(vcc, environment.type, audioId, section, part, resultAudio && resultAudio.fileName ? resultAudio.fileName : null, null, null);
                }
                else if (environment.type === 'ftp') {
                    return GetAudioFileFromFtp(vcc, audioData, environment);
                }
                else if (environment.type === 'sftp') {
                    logger.info('[AudioService::get_audio_transcription] Getting audio from sftp');
                    return GetAudioFileFromSftp(vcc, audioData, environment);
                }
            }
            return true;
        }).catch(function (err) {
            if (typeof err === 'string'
                &&
                    [
                        'INVALID_AUDIO_DATA',
                        'SERVER_ERROR_AUDIO_SYSTEM_SERVICE_NOT_FOUND',
                        'ENVIRONMENT_NOT_FOUND'
                    ].includes(err))
                throw err;
            return null;
        }).then(function (result) {
            logger.info('[AudioService::get_audio_transcription] Receiving audio from sftp');
            if (typeof result !== 'boolean') {
                if (result && result.audioFile && typeof result.audioFile === 'string') {
                    fileNameAudio = result && result.audioFile ? result.audioFile.split('/').pop() : null;
                }
                else {
                    fileNameAudio = null;
                }
                buffer = result && result.buffer ? result.buffer : null;
                if (resultAudio) {
                    if (resultAudio.fileName === null) {
                        var fileName = resultAudio.id;
                        if (resultAudio.section !== '' && resultAudio.part !== '') {
                            fileName += '_';
                            fileName += section;
                            fileName += '_';
                            fileName += part;
                        }
                        fileName += '.json';
                        resultAudio.fileName = fileName;
                    }
                    var regex = /\.(mp3|wav|json)$/;
                    jsonFileName = resultAudio.fileName.replace(regex, '.json');
                    fileNameAnalysisSemantic = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                    fileNameAnalysisSemantic += '/';
                    fileNameAnalysisSemantic += vcc;
                    fileNameAnalysisSemantic += '/';
                    fileNameAnalysisSemantic += jsonFileName;
                    fileNameAnalysisSemanticView = config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER;
                    fileNameAnalysisSemanticView += '/';
                    fileNameAnalysisSemanticView += vcc;
                    fileNameAnalysisSemanticView += '/';
                    fileNameAnalysisSemanticView += jsonFileName;
                    logger.info('[AudioService::get_audio_transcription] Getting resultAudio:', resultAudio, 'before ExportAudioTranscription with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
                    logger.debug('[AudioService::get_audio_transcription] Searching raw transcription:', fileNameAnalysisSemantic);
                    logger.debug('[AudioService::get_audio_transcription] Searching view transcription:', fileNameAnalysisSemanticView);
                    if (resultAudio.init === null && resultAudio.isTranscribed === 1 && !resultAudio.isCancel) {
                        return ExportAudioTranscription(vcc, resultAudio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView);
                    }
                }
            }
            return true;
        }).then(function (result) {
            logger.info('[AudioService::get_audio_transcription] Getting result:', result, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
            if (fileNameAnalysisSemanticView && fileNameAnalysisSemanticView.length > 0) {
                return file_manager_1.ReadFileContent(fileNameAnalysisSemanticView, 'utf-8');
            }
            return '';
        }).then(function (result) {
            if (result && result.length > 0) {
                _resultReadFileContent = JSON.parse(result);
                var diarizationAnalysis = new diarization_analysis_implementation_1.DiarizationAnalysisImplementation();
                var phrases = diarizationAnalysis.GeneratePhrases(_resultReadFileContent.words, 1000000);
                _resultReadFileContent.chat = phrases.allPhrases;
            }
            if (resultAudio && resultAudio.processId) {
                var transcriptionDTO = {
                    vcc: vcc,
                    processId: resultAudio.processId,
                    id: resultAudio.id,
                    section: resultAudio.section,
                    part: resultAudio.part
                };
                return mssql_service_1.MSSql.GetTranscription(transcriptionDTO);
            }
            return true;
        }).then(function (result) {
            result = (result && result.StatusCodeError) ? null : result;
            if (typeof result !== 'boolean') {
                _transcriptionQualification = result ? result.value : null;
            }
            if (environment.type === 'allegro' || environment.type === 'i6') {
                return GetVideoFromAllegro(vcc, audioId);
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then(function (result) {
            var data = {
                audioFile: fileNameAudio,
                channelsChanged: resultAudio && resultAudio.channelsChanged ? resultAudio.channelsChanged : false,
                buffer: buffer,
                resultTranscription: _resultReadFileContent,
                transcriptionQualification: _transcriptionQualification,
                video: result ? result : []
            };
            if (res.headersSent)
                return;
            logger.debug('[AudioService::get_audio_transcription] Response data:', fileNameAnalysisSemanticView, data);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AudioService::get_audio_transcription] Getting errorMsg:', errorMsg, 'with req.params:', req.params, 'with vcc:', vcc, 'with resultAudio:', resultAudio);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_process_audio_summary/:processId', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.processId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetProcessAudioSummary(vcc, processId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/process/audio/get_audio_directory_count/:process_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var processId = req.params.process_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return file_manager_1.GetFolderFiles(config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + processId, ['csv'], ['wav', 'mp3'], true);
        }).then(function (resultGetFolderFiles) {
            if (res.headersSent)
                return;
            resultGetFolderFiles = JSON.parse(JSON.stringify(resultGetFolderFiles));
            resultGetFolderFiles = resultGetFolderFiles.filter(function (x) { return x.type !== 'directory'; });
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGetFolderFiles.length });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/process/audio/save_transcription', function (req, res, next) {
        logger.info('[AudioService::save_transcription] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var newTranscriptionDTO = (req.body || {});
        newTranscriptionDTO.vcc = vcc;
        newTranscriptionDTO.userId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.SaveTranscription(newTranscriptionDTO);
        }).then(function (resultSaveTranscription) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultSaveTranscription });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.AudioService = AudioService;
function SaveAudio(vcc, audio) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveAudio(vcc, audio.id, audio.section, audio.part, audio.processId, audio.number, audio.gender, audio.silenceTime, audio.durationTime, audio.category1, audio.category2, audio.category3, audio.description, audio.campaign, audio.agent, audio.type, audio.init));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function UpdateAudioInit(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView) {
    var fileContent = '';
    return Promise.resolve()
        .then(function (result) {
        logger.info('[AudioService::UpdateAudioInit] UpdateAudioInit: ', audio);
        return file_manager_1.ReadFileContent(fileNameAnalysisSemantic, 'utf-8');
    }).catch(function (err) {
        return GetIdathaDocument(vcc, audio, fileNameAnalysisSemantic);
    })
        .then(function (result) {
        return file_manager_1.ReadFileContent(fileNameAnalysisSemantic, 'utf-8');
    })
        .then(function (resultReadFileContent) {
        fileContent = resultReadFileContent;
        logger.info('[AudioService::UpdateAudioInit] Received ReadFileContent\'s result: ' + JSON.stringify(resultReadFileContent));
        var result = JSON.parse(fileContent).words;
        audio.init = result.length > 0 ? result[0].speaker : null;
        return SaveAudio(vcc, audio);
    }).then(function (result) {
        var resultSortJson = SortJson(fileContent);
        fileContent = resultSortJson;
        return file_manager_1.WriteFileContent(fileNameAnalysisSemanticView, resultSortJson, 'utf-8');
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar UpdateAudioInit: ' + errMsg);
        throw errMsg;
    });
}
function SortJson(jsonobj) {
    var outlength;
    var inlength;
    var i;
    var audioInit;
    var obj = {};
    obj.transcription = [];
    obj.summarySpeechData = {};
    obj.confidence = {};
    obj.init = '';
    var json = JSON.parse(jsonobj);
    audioInit = json.words[0].speaker;
    obj.init = audioInit;
    obj.nlp = {};
    outlength = json.out_sentences.length;
    inlength = json.in_sentences.length;
    if (outlength >= inlength) {
        for (i in json.out_sentences) {
            if (i < inlength && inlength > 0) {
                if (audioInit === 'out' && inlength > 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i].text,
                        in: json.in_sentences[i] === 'undefined' ? '' : json.in_sentences[i].text
                    });
                }
                else if (audioInit === 'in' && inlength > 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i].text,
                        out: json.out_sentences[i].text
                    });
                }
            }
            else {
                if (audioInit === 'out' && inlength >= 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text,
                        in: ''
                    });
                }
                else if (audioInit === 'in' && inlength >= 0) {
                    obj.transcription.push({
                        in: '',
                        out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text
                    });
                }
            }
        }
    }
    else if (outlength <= inlength) {
        for (i in json.in_sentences) {
            if (i < outlength && outlength > 0) {
                if (audioInit === 'out' && outlength > 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i].text,
                        in: json.in_sentences[i].text
                    });
                }
                else if (audioInit === 'in' && outlength > 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i].text,
                        out: json.out_sentences[i].text
                    });
                }
            }
            else {
                if (audioInit === 'out' && inlength >= 0) {
                    obj.transcription.push({
                        out: '',
                        in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                    });
                }
                else if (audioInit === 'in' && inlength >= 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                        out: ''
                    });
                }
            }
        }
    }
    obj.nlp.in = json.pnl_scores_in ? json.pnl_scores_in : 0;
    obj.nlp.out = json.pnl_scores_out ? json.pnl_scores_out : 0;
    var dataTotalIn = {
        duration: 0,
        silence: 0
    }, dataTotalOut = {
        duration: 0,
        silence: 0
    }, data = json.words, len = data.length, word, inDuration = 0, inSilence = 0, outDuration = 0, outSilence = 0;
    for (i = 0; i < len; i++) {
        word = data[i];
        if (word.speaker === 'in') {
            inDuration += word.duration;
            inSilence += word.silence;
        }
        else {
            outDuration += word.duration;
            outSilence += word.silence;
        }
    }
    dataTotalIn.duration = inDuration.toFixed(2) * 1;
    dataTotalIn.silence = inSilence.toFixed(2) * 1;
    dataTotalOut.duration = outDuration.toFixed(2) * 1;
    dataTotalOut.silence = outSilence.toFixed(2) * 1;
    obj.summarySpeechData = {
        dataTotalIn: dataTotalIn,
        dataTotalOut: dataTotalOut
    };
    obj.duration = json.duration;
    obj.words = json.words;
    obj.topics = json.topics;
    obj.confidence.in_confidence = json.in_confidence ? json.in_confidence : 0;
    obj.confidence.out_confidence = json.out_confidence ? json.out_confidence : 0;
    return JSON.stringify(obj);
}
function ExportAudioTranscription(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView) {
    return Promise.resolve()
        .then(function (result) {
        return file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_FOLDER + '/' + vcc);
    }).then(function (result) {
        return file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER + '/' + vcc);
    }).then(function () {
        return file_manager_1.ExistInFileSystem(fileNameAnalysisSemanticView);
    }).then(function (resultExistInFileSystem) {
        if (!resultExistInFileSystem) {
            return UpdateAudioInit(vcc, audio, fileNameAnalysisSemantic, fileNameAnalysisSemanticView);
        }
        return Promise.resolve(true);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[AudioService::ExportAudioTranscription] Getting error:', err, 'with vcc:', vcc, 'with audio:', audio, 'fileNameAnalysisSemantic:', fileNameAnalysisSemantic, 'fileNameAnalysisSemanticView:', fileNameAnalysisSemanticView);
        throw errMsg;
    });
}
function ExternalRequest(url) {
    var options = {
        uri: url,
        encoding: null,
        timeout: 30000,
        responseType: 'stream'
    };
    logger.info('[audio.service::ExternalRequest] Option:', options);
    return request_shared_1.SimpleGetRequest(options)
        .then(function (result) {
        logger.info('[audio.service::ExternalRequest] OK');
        var buffer = null;
        var bufs = [];
        return new Promise(function (resolve, reject) {
            result.on('error', function (err) {
                logger.error('[audio.service::ExternalRequest] SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
            });
            result.once('close', function () {
                try {
                    buffer = Buffer.concat(bufs);
                    return resolve(buffer);
                }
                catch (e) {
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                }
            });
            result.on('data', function (d) {
                try {
                    bufs.push(d);
                }
                catch (e) {
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                }
            });
        });
    })
        .catch(function (err) {
        logger.info('[audio.service::ExternalRequest] SERVER_ERROR_UNKNOWN_MESSAGE');
        return Promise.reject(err);
    });
}
function getTimes(startTime, endTime) {
    var startHour, startMinute, endHour, endMinute;
    if (startTime && startTime.length > 0) {
        var arrayStartTime = startTime.split(':');
        if (arrayStartTime && arrayStartTime.length > 0) {
            startHour = arrayStartTime[0];
            startMinute = arrayStartTime[1];
        }
    }
    else {
        startHour = 0;
        startMinute = 0;
    }
    if (endTime && endTime.length > 0) {
        var arrayEndTime = endTime.split(':');
        if (arrayEndTime && arrayEndTime.length > 0) {
            endHour = arrayEndTime[0];
            endMinute = arrayEndTime[1];
        }
    }
    else {
        endHour = 23;
        endMinute = 59;
    }
    return {
        startTime: {
            hour: startHour,
            minute: startMinute,
        },
        endTime: {
            hour: endHour,
            minute: endMinute,
        }
    };
}
function GetIdathaDocument(vcc, audio, fileName) {
    var idatha = {
        user: config_1.IDATHA_USER,
        password: config_1.IDATHA_PASSWORD,
        ip: config_1.IDATHA_IP,
        httpPort: config_1.IDATHA_HTTP_PORT,
        sshPort: config_1.IDATHA_SSH_PORT,
    };
    var userName = '';
    var password = '';
    var accessToken = '';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.GetUserAccessTokenByRealm(vcc, vcc);
    })
        .then(function (resultUserAccessTokenByRealm) {
        if (resultUserAccessTokenByRealm.accessToken) {
            accessToken = resultUserAccessTokenByRealm.accessToken;
            var currentDate = new Date(), createdDate = new Date(resultUserAccessTokenByRealm.createdDate), diffDays = Math.round(Math.abs((currentDate.getTime() - createdDate.getTime()) / (24 * 60 * 60 * 1000)));
            if (diffDays >= 7) {
                return idatha_service_1.IdathaLogin(idatha, vcc);
            }
            else {
                accessToken = resultUserAccessTokenByRealm.accessToken;
                return accessToken;
            }
        }
        else {
            return resultUserAccessTokenByRealm;
        }
    })
        .then(function (result) {
        return IdathaDocumentWords(idatha, accessToken, audio.id, audio.section, audio.part);
    }).then(function (resultIdathaDocumentWords) {
        var regex = /\bNaN\b/g;
        var data;
        data = JSON.stringify(resultIdathaDocumentWords.response);
        if (regex.test(resultIdathaDocumentWords.response))
            data = resultIdathaDocumentWords.response.replace(regex, 'null');
        if (resultIdathaDocumentWords.response)
            return file_manager_1.WriteFileContent(fileName, data, 'utf-8');
        return true;
    });
}
exports.GetIdathaDocument = GetIdathaDocument;
function IdathaDocumentWords(idatha, accessToken, audioId, section, part) {
    var url = "http://" + idatha.ip + ":" + idatha.httpPort + "/api/query/document/" + audioId;
    if (section !== '' && part !== '0') {
        url += '_';
        url += section;
        url += '_';
        url += part;
    }
    var options = {
        uri: url,
        headers: {
            'Authorization': 'Bearer ' + accessToken
        }
    };
    logger.info('[audio.service::IdathaDocumentWords] Options: ', options);
    return request_shared_1.SimpleGetRequest(options)
        .then(function (response) {
        var content = { response: response, type: 'document' };
        return Promise.resolve(content);
    }).catch(function (err) {
        options = {
            uri: 'http://' + idatha.ip + ':' + idatha.httpPort + '/api/query/document/' + audioId,
            headers: {
                'Authorization': 'Bearer ' + accessToken
            }
        };
        logger.info('[audio.service::IdathaDocumentWords] Options - catch: ', options);
        return request_shared_1.SimpleGetRequest(options)
            .then(function (response) {
            var content = { response: response, type: 'document' };
            return Promise.resolve(content);
        }).catch(function (err) {
            logger.error('[audio.service::IdathaDocumentWords] Error on: ' + err.message);
            return Promise.reject(err);
        });
    });
}
exports.IdathaDocumentWords = IdathaDocumentWords;
function GetAudioFilePossiblePaths(audioId, audioRelativePath, audioFileName, environment) {
    var filePaths = [];
    if (audioRelativePath === null && audioFileName !== null) {
        filePaths.push(environment.ftpDirectory + '/' + audioFileName);
        filePaths.push(environment.ftpDirectory + '/' + audioId + '.wav');
        filePaths.push(environment.ftpDirectory + '/' + audioId + '.mp3');
    }
    else if (audioRelativePath !== null && audioFileName === null) {
        filePaths.push(audioRelativePath + '/' + audioId + '.wav');
        filePaths.push(audioRelativePath + '/' + audioId + '.mp3');
        filePaths.push(environment.ftpDirectory + '/' + audioId + '.wav');
        filePaths.push(environment.ftpDirectory + '/' + audioId + '.mp3');
    }
    else if (audioRelativePath !== null && audioFileName !== null) {
        filePaths.push(audioRelativePath + '/' + audioFileName);
        filePaths.push(environment.ftpDirectory + '/' + audioFileName);
        filePaths.push(audioRelativePath + '/' + audioId + '.wav');
        filePaths.push(audioRelativePath + '/' + audioId + '.mp3');
        filePaths.push(environment.ftpDirectory + '/' + audioId + '.wav');
        filePaths.push(environment.ftpDirectory + '/' + audioId + '.mp3');
    }
    else {
        filePaths.push(environment.ftpDirectory + '/' + audioId + '.wav');
        filePaths.push(environment.ftpDirectory + '/' + audioId + '.mp3');
    }
    return filePaths;
}
function GetAudioFileFromFtp(vcc, audioData, environment) {
    var audioId = audioData.audioId;
    var audioRelativePath = audioData.relativePath;
    var audioFileName = audioData.fileName;
    var filePaths = GetAudioFilePossiblePaths(audioId, audioRelativePath, audioFileName, environment);
    var environmentConfig = {
        host: environment.apiHost,
        port: environment.apiPort,
        user: environment.apiUser,
        password: environment.apiPassword,
        connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
        pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
    };
    var genericResult = {};
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return new Promise(function (resolve, reject) {
            var clientFtp = new Client();
            clientFtp.connect(environmentConfig);
            if (clientFtp) {
                clientFtp.on('greeting', function (message) {
                    logger.info('[AudioService::GetAudioFileFromFtp] Greeting event message:', message);
                });
                clientFtp.on('ready', function () {
                    SearchAudioFileOnFtp(clientFtp, filePaths, genericResult, resolve, reject);
                });
                clientFtp.on('end', function () {
                    var eventMessage = 'FTP_CONNECTION_ENDED';
                    logger.warn('[AudioService::GetAudioFileFromFtp] getting eventMessage:', eventMessage);
                    return resolve(eventMessage);
                });
                clientFtp.on('close', function (hadError) {
                    var eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                    logger.warn('[AudioService::GetAudioFileFromFtp] hadError:', hadError);
                    return resolve(eventMessage);
                });
                clientFtp.on('error', function (err) {
                    logger.error('[AudioService::GetAudioFileFromFtp] NOT_FTP_CLIENT', err);
                    return reject('NOT_FTP_CLIENT');
                });
            }
            else {
                logger.error('[AudioService::GetAudioFileFromFtp] Can not create FTP Client');
                return reject('SERVER_ERROR_CANNOT_CREATE_FTP_CLIENT');
            }
        });
    })
        .then(function (result) {
        return Promise.resolve({
            audioFile: result,
            buffer: genericResult.extraContent
        });
    })
        .catch(function (error) {
        return Promise.reject(error);
    });
}
exports.GetAudioFileFromFtp = GetAudioFileFromFtp;
function SearchAudioFileOnFtp(clientFtp, filePaths, genericResult, resolve, reject) {
    var ftpFile = filePaths.shift();
    logger.debug('[AudioService::SearchAudioFileOnFtp] Searching:', ftpFile);
    clientFtp.get(ftpFile, function (err, stream) {
        if (!err) {
            genericResult.status = true;
            var bufs_1 = [];
            stream.once('close', function () {
                clientFtp.end();
                genericResult.extraContent = Buffer.concat(bufs_1);
                resolve(ftpFile.split('/').pop());
            });
            stream.on('data', function (d) {
                bufs_1.push(d);
            });
        }
        else {
            logger.error('[AudioService::SearchAudioFileOnFtp] Audiofile not found on:', ftpFile);
            if (filePaths.length) {
                SearchAudioFileOnFtp(clientFtp, filePaths, genericResult, resolve, reject);
            }
            else {
                clientFtp.end();
                genericResult.status = false;
                genericResult.extraContent = 'SERVER_ERROR_AUDIO_FILE_GET_FTP';
                reject(genericResult);
            }
        }
    });
}
function GetAudioFileFromSftp(vcc, audioData, environment) {
    var audioId = audioData.audioId;
    var audioRelativePath = audioData.relativePath;
    var audioFileName = audioData.fileName;
    var filePaths = GetAudioFilePossiblePaths(audioId, audioRelativePath, audioFileName, environment);
    var environmentConfig = {
        host: environment.apiHost,
        port: environment.apiPort,
        username: environment.apiUser,
        password: environment.apiPassword,
        connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
        pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
    };
    var audioFile = null;
    var genericResult = {};
    var ssh2 = new SSH2Promise(environmentConfig);
    var sftp;
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        if (ssh2) {
            sftp = ssh2.sftp();
            return SearchAudioFileOnSftp(sftp, filePaths, genericResult);
        }
        else {
            logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_NOT_SFTP_CLIENT');
            throw 'SERVER_ERROR_NOT_SFTP_CLIENT';
        }
    })
        .then(function (result) {
        if (genericResult.status === true) {
            logger.info('[AudioService::GetAudioFileFromSftp] Resolving general result');
            return Promise.resolve({
                audioFile: result,
                buffer: genericResult.extraContent
            });
        }
        else {
            logger.error('[AudioService::GetAudioFileFromSftp] SERVER_ERROR_GET_AUDIO_FROM_SFTP_FAILED');
            throw 'SERVER_ERROR_GET_AUDIO_FROM_SFTP_FAILED';
        }
    })
        .catch(function (err) {
        genericResult.status = false;
        genericResult.error = err;
        logger.error('[AudioService::GetAudioFileFromSftp] General error: ', err);
        return Promise.reject({
            audioFile: audioFile,
            buffer: genericResult.error
        });
    });
}
exports.GetAudioFileFromSftp = GetAudioFileFromSftp;
function SearchAudioFileOnSftp(sftp, filePaths, genericResult) {
    var sftpFile = filePaths.shift();
    logger.debug('[AudioService::SearchAudioFileOnSftp] Searching:', sftpFile);
    return sftp.stat(sftpFile)
        .then(function (result) {
        return sftp.createReadStream(sftpFile, { flags: 'r', mode: 438 })
            .then(function (stream) {
            if (stream) {
                genericResult.status = true;
                var bufs_2 = [];
                return new Promise(function (resolve, reject) {
                    stream.on('error', function (err) {
                        logger.error('[AudioService::SearchAudioFileOnSftp] SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                        reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                    });
                    stream.once('close', function () {
                        try {
                            sftp.close();
                            genericResult.extraContent = Buffer.concat(bufs_2);
                            return resolve(sftpFile.split('/').pop());
                        }
                        catch (e) {
                            return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                        }
                    });
                    stream.on('data', function (d) {
                        try {
                            bufs_2.push(d);
                        }
                        catch (e) {
                            return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                        }
                    });
                });
            }
            else {
                logger.error('[AudioService::SearchAudioFileOnSftp] SERVER_ERROR_GET_AUDIO_STREAM_FAILED');
                throw 'SERVER_ERROR_GET_AUDIO_STREAM_FAILED';
            }
        })
            .catch(function (err) {
            throw err;
        });
    })
        .catch(function (err) {
        logger.warn('[AudioService::SearchAudioFileOnSftp] Audiofile not found on:', sftpFile);
        if (filePaths.length) {
            return SearchAudioFileOnSftp(sftp, filePaths, genericResult);
        }
        else {
            return Promise.resolve(null);
        }
    });
}
function GetAudioFileFromIntegrated(vcc, environmentType, interactionId, section, part, fileName, ext, target) {
    if (!ext)
        ext = '.wav';
    if (!target)
        target = 'LOCAL';
    var fileNameAudio = null;
    if (fileName === null) {
        fileNameAudio = interactionId;
        fileNameAudio += '_';
        fileNameAudio += section;
        fileNameAudio += '_';
        fileNameAudio += part;
        fileNameAudio += ext;
    }
    if (fileNameAudio !== null) {
        var filePath = '';
        if (target === 'LOCAL') {
            filePath = part !== null && !isNaN(part) && part > 0 ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
            filePath += '/';
            filePath += vcc;
            filePath += '/';
        }
        fileNameAudio = filePath + fileNameAudio;
    }
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        switch (target) {
            case 'LOCAL': return file_manager_1.BufferFile(fileNameAudio);
            case 'REMOTE': return GetAudioFromIntegrated(vcc, environmentType, interactionId, section, part, fileNameAudio);
            default: throw 'SERVER_ERROR_INVALID_TARGET';
        }
    }).catch(function (error) {
        logger.error('[AudioService::GetAudioFileFromIntegrated] Tried: ', fileNameAudio);
        if (ext === '.wav' && target === 'LOCAL' && (fileName === null || fileName.includes('.mp3')))
            ext = '.mp3';
        else if (ext === '.mp3' && target === 'LOCAL' && (fileName === null || fileName.includes('.wav'))) {
            ext = '.wav';
            target = 'REMOTE';
        }
        else if (ext === '.wav' && target === 'REMOTE' && (fileName === null || fileName.includes('.mp3')))
            ext = '.mp3';
        else if (ext === '.mp3' && target === 'REMOTE')
            throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
        fileNameAudio = null;
        return GetAudioFileFromIntegrated(vcc, environmentType, interactionId, section, part, null, ext, target);
    }).then(function (result) {
        if (!result)
            'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
        var res = {};
        if (fileNameAudio) {
            logger.info('[AudioService::GetAudioFileFromIntegrated] Found on: ', fileNameAudio);
            res = {
                audioFile: fileNameAudio,
                buffer: result
            };
        }
        else {
            res = result;
        }
        return Promise.resolve(res);
    }).catch(function (error) {
        return Promise.reject(error);
    });
}
exports.GetAudioFileFromIntegrated = GetAudioFileFromIntegrated;
function GetAudioFromIntegrated(vcc, environmentType, interactionId, section, part, fileNameAudio) {
    var audioFilePath = null;
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        switch (environmentType) {
            case 'allegro':
                audioFilePath = "https://" + integration_config_1.IP_MW + ":" + integration_config_1.PORT_MW + "/inconcert/apps/mediaserver/action/download/repository/" + interactionId + "/" + section + "/" + part + "/";
                fileNameAudio = audioFilePath + fileNameAudio;
                return ExternalRequest(fileNameAudio);
            case 'i6':
                return http_request_occ_1.ApiI6GetInteractionFile(vcc, interactionId, section, part);
            default:
                throw 'SERVER_ERROR_UNKNOWN_ENVIRONMENT_TYPE';
        }
    })
        .catch(function (err) {
        logger.info('[audioService::GetAudioFromIntegrated] Catch: ', err);
        return Promise.reject(err);
    });
}
function GetVideoFromAllegro(vcc, interactionId) {
    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
    var myVCC = null;
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return vcc_service_1.GetVCC(vcc, vcc);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_VCC_NOT_FOUND';
        myVCC = result;
        if (myVCC.type === 'OCC' || myVCC.type === 'I6') {
            return mssql_service_1.MSSql.GetAllegroInteractionVideos(integration_config_1.IP_BD, vcc, flagAllegro, interactionId);
        }
        else
            throw 'SERVER_ERROR_VCC_TYPE_NOT_SUPPORTED';
    })
        .then(function (result) {
        result.map(function (r) {
            r.path = 'http://';
            r.path += integration_config_1.DNS;
            r.path += '/inconcert/apps/mediaserver/action/download/repository/';
            r.path += r.id;
            r.path += '/';
            r.path += r.section;
            r.path += '/';
            r.path += r.part;
            r.path += '/';
            r.path += r.id;
            r.path += '_';
            r.path += r.section;
            r.path += '_';
            r.path += r.part;
            r.path += '.';
            r.path += r.format;
        });
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.info('[audioService::GetVideoFromAllegro] Catch: ', err);
        return Promise.resolve([]);
    });
}
