"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompareLicenses = exports.AuthService = void 0;
var moment = require("moment");
var jwt = require("jsonwebtoken");
var log4js = require("log4js");
var jwt_config_1 = require("../config/jwt.config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var user_service_1 = require("./user.service");
var role_service_1 = require("./role.service");
var promises_shared_1 = require("../shared/promises.shared");
var billing_implementation_1 = require("../implementation/billing.implementation");
var vcc_service_1 = require("./vcc.service");
var logger = log4js.getLogger('ServerScripts');
function AuthService(app) {
    app.post('/auth/login', function (req, res) {
        var vcc = res.locals.vcc;
        var nowUtc = moment().utc().toDate();
        var credential = req.body.userId;
        var credentials = credential.split('@');
        var userId = credentials.length > 0 ? credentials[0] : '';
        vcc = credentials[1];
        var password = req.body.password;
        if (credential === 'admin')
            vcc = 'system';
        var session;
        var user;
        var role;
        var vccSpeech = [];
        var vccCenter = {};
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!userId || !password)
                throw 'AUTH_SERVER_ERROR_MISSING_USER_OR_PASSWORD';
            userId = userId.toLowerCase();
            return mssql_service_1.MSSql.ValidateUserLogin(vcc, userId, password);
        }).then(function (resultValidateUserLogin) {
            if (!resultValidateUserLogin)
                throw 'AUTH_SERVER_ERROR_WRONG_USER_OR_PASSWORD';
            return mssql_service_1.MSSql.GetVCCLicense(vcc);
        }).then(function (result) {
            if (credential === 'admin') {
                return Promise.resolve(null);
            }
            else {
                vccSpeech = result;
                var billing = new billing_implementation_1.BillingImplementation();
                return billing.GetVCCStatusDetail(vcc);
            }
        }).then(function (result) {
            var compare;
            if (credential !== 'admin') {
                if (typeof result == 'string') {
                    logger.info('[AuthService::/auth/login] Result afet Billing GetVCCStatus' + result);
                    throw result;
                }
                if (result.error && result.message.includes('ECONNREFUSED')) {
                    throw 'ERROR_FROM_EXTERNAL_SERVER';
                }
                if (result && !result.error && typeof result != 'string') {
                    logger.info('[AuthService::login] Result afet Billing GetVCCStatus' + result);
                    vccCenter = result;
                    compare = CompareLicenses(vccSpeech, vccCenter, userId, nowUtc);
                    if (!result.products.find(function (rp) { return rp.valid === 1; }))
                        throw 'LICENSE_EXPIRED';
                }
            }
            logger.info('[AuthService::/auth/login] Compare VCC Licenses between SPEECH DB and BILLING DB , match : ' + compare);
            session = jwt.sign({ userId: userId, vcc: vcc }, jwt_config_1.JWT_SECRET, { expiresIn: jwt_config_1.JWT_EXPIRATION_TIME });
            return redis_service_1.Redis.SaveSession(vcc, userId, session);
        }).then(function (resultSaveSession) {
            if (!resultSaveSession)
                throw 'AUTH_SERVER_ERROR_CANNOT_SAVE_SESSION';
            return user_service_1.GetUser(vcc, userId);
        }).then(function (resultUser) {
            if (!resultUser)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_SESSION';
            user = resultUser;
            return mssql_service_1.MSSql.InsertUserHistory(vcc, userId, user.role, 'Login', nowUtc, session);
        }).then(function (resultInsertUserHistory) {
            if (!resultInsertUserHistory)
                throw 'AUTH_SERVER_ERROR_CANNOT_SAVE_SESSION';
            return role_service_1.GetRole(vcc, user.role);
        }).then(function (resultRole) {
            if (!resultRole)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_SESSION';
            role = resultRole;
            if (!session || !user || !role)
                throw 'AUTH_SERVER_ERROR_ON_START_SESSION';
            var data = {
                session: session,
                loggedUser: user,
                loggedUserRole: role
            };
            res.cookie('Session', session, { httpOnly: true });
            logger.info('[AuthService::/auth/login] Se inicio sesion ' + session + ' para el usuario ' + userId + '@' + vcc);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Login failed');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AuthService::/auth/login] Error logging for user ' + userId + ' : ' + errorMsg);
            if (res.headersSent)
                return;
            res.status(401).json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/auth/logout', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var user = res.locals.user;
        var nowUtc = moment().utc().toDate();
        var session = req.body.session || res.locals.session || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return redis_service_1.Redis.DeleteSession(vcc, userId);
        }).then(function (result) {
            return redis_service_1.Redis.DeleteUserById(vcc, userId);
        }).then(function (result) {
            return mssql_service_1.MSSql.InsertUserHistory(vcc, userId, user.role, 'Logout', nowUtc, session);
        }).then(function (result) {
            res.clearCookie('Session');
            logger.info('Se finalizo sesion ' + session + ' para el usuario ' + userId + '@' + vcc);
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'AUTH_SERVER_ERROR_ON_END_SESSION'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Logout failed');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al finalizar sesion para el usuario ' + userId + ' : ' + errorMsg);
            if (res.headersSent)
                return;
            res.status(401).json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/auth/get_user_session_data', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vccName = res.locals.vcc;
        var userId = res.locals.userId;
        userId = (userId || '').toLowerCase();
        var vcc;
        var session;
        var user;
        var role;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return vcc_service_1.GetVCC(vccName, vccName);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_VCC_NOT_FOUND';
            vcc = result;
            return redis_service_1.Redis.GetSession(vccName, userId);
        }).then(function (resultSession) {
            if (!resultSession)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_SESSION';
            session = resultSession;
            return user_service_1.GetUser(vcc.id, userId);
        }).then(function (resultUser) {
            if (!resultUser)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_USER_DATA';
            user = resultUser;
            return role_service_1.GetRole(vcc.id, user.role);
        }).then(function (resultRole) {
            if (!resultRole)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_USER_DATA';
            role = resultRole;
            if (!session || !user || !role)
                throw 'AUTH_SERVER_ERROR_ON_START_SESSION';
            var data = {
                session: session,
                loggedUser: user,
                loggedUserRole: role,
                vcc: vcc.id,
                cognitiveEngine: vcc.cognitiveEngine
            };
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AuthService = AuthService;
function CompareLicenses(locals, external, user, date) {
    var status = true;
    if (external.products.length > 0) {
        locals.forEach(function (x) {
            external.products.forEach(function (y) {
                if (x.product === y.product) {
                    if (x.status != y.valid) {
                        status = false;
                        mssql_service_1.MSSql.SaveVCCLicense(x.product, y.valid, x.vcc, x.createdDate, x.createdByUserId, date, user);
                    }
                }
            });
        });
    }
    else {
        locals.forEach(function (x) {
            mssql_service_1.MSSql.SaveVCCLicense(x.product, 0, x.vcc, x.createdDate, x.createdByUserId, date, user);
        });
    }
    return status;
}
exports.CompareLicenses = CompareLicenses;
