"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CountryService = void 0;
var log4js = require("log4js");
var wc = require('world-countries');
var logger = log4js.getLogger('ServerScripts');
function CountryService(app) {
    app.get('/api/country/get_countries/:language', function (req, res, next) {
        var language = req.params.language;
        var iso6393LanguageCode = (language === 'es' ? 'spa' : (language === 'pt' ? 'por' : (language === 'fr' ? 'fra' : ('eng'))));
        var countryCodes = wc
            .map(function (country) {
            return {
                code: country.cca2 || null,
                cca2: country.cca2 || null,
                cca3: country.cca3 || null,
                name: (iso6393LanguageCode === 'eng' ?
                    (country.name && country.name.common) || null :
                    (country.translations && country.translations[iso6393LanguageCode] && country.translations[iso6393LanguageCode].common) || (country.name && country.name.common) || null)
            };
        }).filter(function (country) {
            if (country.code && country.name) {
                return true;
            }
            else {
                return false;
            }
        });
        if (res.headersSent)
            return;
        res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: countryCodes });
    });
    app.get('/api/country/get_countries_phone_codes/:language', function (req, res, next) {
        var language = req.params.language;
        var iso6393LanguageCode = (language === 'es' ? 'spa' : (language === 'pt' ? 'por' : (language === 'fr' ? 'fra' : ('eng'))));
        var countryCodes = wc
            .map(function (country) {
            return {
                code: country.cca2 || null,
                cca2: country.cca2 || null,
                cca3: country.cca3 || null,
                phoneCode: (country.callingCode && country.callingCode.length > 0 && country.callingCode[0]) || null,
                name: (iso6393LanguageCode === 'eng' ?
                    (country.name && country.name.common) || null :
                    (country.translations && country.translations[iso6393LanguageCode] && country.translations[iso6393LanguageCode].common) || (country.name && country.name.common) || null)
            };
        }).filter(function (country) {
            if (country.code && country.phoneCode && country.name) {
                return true;
            }
            else {
                return false;
            }
        });
        if (res.headersSent)
            return;
        res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: countryCodes });
    });
}
exports.CountryService = CountryService;
