"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveGenericQuerys = exports.SaveGenericQuery = exports.GenericQueryService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var idatha_service_1 = require("./idatha.service");
var request_shared_1 = require("../shared/request.shared");
var semantic_query_implementation_1 = require("../implementation/semantic-query.implementation");
var logger = log4js.getLogger('ServerScripts');
function GenericQueryService(app) {
    app.get('/api/generic_query/get_generic_queries/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[GenericQueryService::/api/generic_query/get_generic_queries] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        logger.info('[GenericQueryService::/api/generic_query/get_generic_queries] Received vcc: ' + vcc);
        logger.info('[GenericQueryService::/api/generic_query/get_generic_queries] Received search_criteria: ' + search_criteria);
        logger.info('[GenericQueryService::/api/generic_query/get_generic_queries] Received search_expression: ' + search_expression);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[GenericQueryService::/api/generic_query/get_generic_queries] Calling GetGenericQueries SP');
            return mssql_service_1.MSSql.GetGenericQueries(vcc, search_criteria, search_expression);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/get_generic_queries] Received result from GetGenericQueries SP: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/get_generic_queries] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/generic_query/get_generic_query/:name', function (req, res, next) {
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var name = req.params.name;
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Received vcc: ' + vcc);
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Received query name: ' + name);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!name)
                throw 'MISSING_NAME_PARAMETER';
            logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Calling GetGenericQueryValidate SP');
            return mssql_service_1.MSSql.GetGenericQueryValidate(vcc, name);
        }).then(function (resultGenericQuery) {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Received result from GetGenericQueryValidate SP: ' + resultGenericQuery.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGenericQuery });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/get_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/generic_query/get_generic_query_usage/:name', function (req, res, next) {
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var name = req.params.name;
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Received vcc: ' + vcc, 'name:', name);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!name)
                throw 'MISSING_NAME_PARAMETER';
            logger.info('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Calling GetGenericQueryUsage SP');
            return mssql_service_1.MSSql.GetGenericQueryUsage(vcc, name);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Received result from GetGenericQueryUsage SP: ' + result);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generic_query/create_generic_query', function (req, res, next) {
        logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newGenericQuery = (req.body || {});
        newGenericQuery.createdDate = nowUtc;
        newGenericQuery.createdByUserId = userId;
        newGenericQuery.lastModifiedDate = nowUtc;
        newGenericQuery.lastModifiedByUserId = userId;
        logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Received vcc: ' + vcc);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Calling GetGenericQuery SP for an existing query with the name: ' + newGenericQuery.name);
            return mssql_service_1.MSSql.GetGenericQuery(vcc, newGenericQuery.name);
        }).then(function (resultGenericQuery) {
            if (resultGenericQuery)
                throw 'GENERIC_QUERY_ALREADY_EXISTS \'' + newGenericQuery.name + '\'.';
            logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Calling SaveGenericQuery function');
            return SaveGenericQuery(vcc, newGenericQuery);
        }).then(function (resultSaveGenericQuery) {
            if (!resultSaveGenericQuery)
                throw 'ERROR_ON_SAVE';
            logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Succed');
            return Promise.resolve(true);
        }).then(function (resultSaveGenericQuery) {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Sending response to client');
            res.json({ status: resultSaveGenericQuery, description: (resultSaveGenericQuery ? 'SERVER_SUCCESS_MESSAGE' : 'GENERIC_QUERY_SAVE_FAILED'), data: resultSaveGenericQuery });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/create_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generic_query/save_generic_query', function (req, res, next) {
        logger.info('[GenericQueryService::/api/generic_query/save_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newGenericQuery = (req.body || {});
        newGenericQuery.createdDate = null;
        newGenericQuery.createdByUserId = null;
        newGenericQuery.lastModifiedDate = nowUtc;
        newGenericQuery.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[GenericQueryService::/api/generic_query/save_generic_query] Calling SaveGenericQuery function');
            return SaveGenericQuery(vcc, newGenericQuery);
        }).then(function (resultSaveGenericQuery) {
            if (!resultSaveGenericQuery)
                throw 'ERROR_ON_SAVE';
            logger.info('[GenericQueryService::/api/generic_query/save_generic_query] Succed');
            return Promise.resolve(true);
        }).then(function (resultSaveGenericQuery) {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/save_generic_query] Sending response to client');
            res.json({ status: resultSaveGenericQuery, description: (resultSaveGenericQuery ? 'SERVER_SUCCESS_MESSAGE' : 'GENERIC_QUERY_SAVE_FAILED'), data: resultSaveGenericQuery });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/save_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generic_query/test_generic_query', function (req, res) {
        logger.info('[GenericQueryService::/api/generic_query/test_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var semanticQuery = new semantic_query_implementation_1.SemanticQueryImplementation();
        var query = req.body.query || '';
        var queryType = null;
        var idatha = {
            httpPort: config_1.IDATHA_HTTP_PORT,
            ip: config_1.IDATHA_IP
        };
        var data = JSON.stringify({
            version: 2,
            from: 0,
            size: 1,
            query: query,
        });
        var bodyParsed = JSON.parse(data.replace(/\\\\\\/g, "\\"));
        promises_shared_1.DummyPromise().then(function (_) {
            return idatha_service_1.IdathaLogin(idatha, config_1.IDATHA_TEST_REALM, true);
        }).then(function (token) {
            var tk = typeof token === 'string' ? token : token.accessToken;
            var options = semanticQuery.GetIdathaRequestOptions(vcc, idatha, tk, null, query, 0, 1, 2);
            queryType = options.queryType;
            logger.info('[GenericQueryService::/api/generic_query/test_generic_query] data :' + JSON.stringify(options));
            return request_shared_1.SimplePostRequest(options.body);
        }).then(function (rawResult) {
            var result = semanticQuery.GetIdathaRequestResponse(queryType, query, rawResult);
            res.json({ status: true, data: result, description: 'QUERY_SYNTAX_OK' });
        }).catch(function (err) {
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/test_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            var errorCode = '';
            if (errorMsg.includes("Syntax")) {
                errorCode = 'QUERY_SYNTAX_ERROR';
            }
            else if (errorMsg.includes("Internal") || errorMsg.includes("500")) {
                errorCode = 'QUERY_INTERNAL_SERVER_ERROR';
            }
            else if (errorMsg.includes("Bad") || errorMsg.includes("401")) {
                errorCode = 'QUERY_BAD_USERNAME_OR_PASSWORD';
            }
            else {
                errorCode = 'CANNOT_PROCESS_REQUEST';
            }
            res.json({ status: false, description: errorCode, error: errorMsg });
        });
    });
    app.post('/api/generic_query/delete_generic_query', function (req, res, next) {
        logger.info('[GenericQueryService::/api/generic_query/delete_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var genericQuery = req.body.genericQuery;
        var name = genericQuery.name;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!name)
                throw 'MISSING_GENERIC_QUERY_NAME_INPUT_PARAMETER';
            logger.info('[GenericQueryService::/api/generic_query/delete_generic_query] Calling DeleteGenericQuery function');
            return mssql_service_1.MSSql.DeleteGenericQuery(vcc, name);
        }).then(function (resultDeleteGenericQuery) {
            if (!resultDeleteGenericQuery)
                throw 'ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(function (resultDeleteGenericQuery) {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/delete_generic_query] Sending response to client');
            res.json({ status: resultDeleteGenericQuery, description: (resultDeleteGenericQuery ? 'SERVER_SUCCESS_MESSAGE' : 'GENERIC_QUERY_DELETE_FAILED'), data: resultDeleteGenericQuery });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/delete_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GenericQueryService = GenericQueryService;
function SaveGenericQuery(vcc, genericQuery) {
    logger.info('[GenericQueryService::/api/generic_query/SaveGenericQuery] Staring function: ' + JSON.stringify(genericQuery));
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveGenericQuery(vcc, genericQuery.name, genericQuery.description, genericQuery.query, genericQuery.rawData, genericQuery.createdDate, genericQuery.createdByUserId, genericQuery.lastModifiedDate, genericQuery.lastModifiedByUserId, genericQuery.version));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveGenericQuery = SaveGenericQuery;
function SaveGenericQuerys(vcc, genericQuerys) {
    logger.info('[GenericQueryService::/api/generic_query/SaveGenericQuery] Staring function: ' + JSON.stringify(genericQuerys));
    var promiseArray = [];
    genericQuerys.forEach(function (genericQuery) {
        promiseArray.push(mssql_service_1.MSSql.SaveGenericQuery(vcc, genericQuery.name, genericQuery.description, genericQuery.query, genericQuery.rawData, genericQuery.createdDate, genericQuery.createdByUserId, genericQuery.lastModifiedDate, genericQuery.lastModifiedByUserId, genericQuery.version));
    });
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveGenericQuerys = SaveGenericQuerys;
