"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveUser = exports.GetUser = exports.UserService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_config_1 = require("../config/mssql.config");
var mssql_service_1 = require("../db/mssql.service");
var http_request_occ_1 = require("../shared/http-request-occ");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var integration_config_1 = require("../config/integration.config");
var bluebird_1 = require("bluebird");
var api_user_implementation_1 = require("../implementation/api-user.implementation");
var logger = log4js.getLogger('ServerScripts');
function UserService(app) {
    app.get('/api/user/get_users/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetUsers(vcc, search_criteria, search_expression);
        }).then(function (users) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: users });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/user/get_user/:user_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = req.params.user_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetUser(vcc, userId);
        }).then(function (resultUser) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/create_user', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newUser = (req.body || {});
        newUser.id = (newUser.id || '').toLowerCase();
        newUser.isSystemUser = false;
        newUser.createdDate = nowUtc;
        newUser.createdByUserId = userId;
        newUser.lastModifiedDate = nowUtc;
        newUser.lastModifiedByUserId = userId;
        var flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return SaveUserIntegration(vcc, newUser, integration_config_1.IP_BD, flag, false);
        })
            .then(function (resultSaveUser) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUser, description: (resultSaveUser ? 'SERVER_SUCCESS_MESSAGE' : resultSaveUser), data: resultSaveUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/save_user', function (req, res, next) {
        logger.info('[UserService::save_user] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newUser = (req.body || {});
        newUser.id = (newUser.id || '').toLowerCase();
        newUser.isSystemUser = false;
        newUser.createdDate = null;
        newUser.createdByUserId = null;
        newUser.lastModifiedDate = nowUtc;
        newUser.lastModifiedByUserId = userId;
        var flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
        logger.info('[UserService::save_user] External database: ' + flag);
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetUser(vcc, newUser.id);
        })
            .then(function (result) {
            if (!result)
                throw 'ERROR_USER_NOT_FOUND';
            if (result.role === 'quality_auditor') {
                logger.info('[UserService::save_user] Checking interactions from Auditor');
                return mssql_service_1.MSSql.GetAuditorsInteractions(vcc, 'id', newUser.id);
            }
            else {
                return bluebird_1.resolve(true);
            }
        })
            .then(function (result) {
            if (newUser.role !== 'quality_auditor'
                && result && result[0] && result[0].uncompleteInteractions && result[0].uncompleteInteractions > 0) {
                throw 'ERROR_USER_PENDING_EVALUATIONS';
            }
            return SaveUserIntegration(vcc, newUser, integration_config_1.IP_BD, flag, true);
        }).then(function (resultSaveUser) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUser, description: (resultSaveUser ? 'SERVER_SUCCESS_MESSAGE' : resultSaveUser), data: resultSaveUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[UserService::save_user] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/delete_user', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var user = req.body.user;
        var userId = user.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return DeleteUser(vcc, userId);
        })
            .then(function (resultDeleteUser) {
            if (typeof resultDeleteUser === 'boolean' && resultDeleteUser === true) {
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: '', data: resultDeleteUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/delete_users', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var users = req.body.users || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!users)
                throw 'MISSING_USERS_INPUT_PARAMETER';
            var promiseArray = [];
            users.forEach(function (user) {
                promiseArray.push(DeleteUser(vcc, user.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteUsers) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: '', data: resultDeleteUsers });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/current_user/get_user', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetUser(vcc, userId);
        }).then(function (resultUser) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/current_user/save_user', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newUser = (req.body || {});
        newUser.id = (newUser.id || '').toLowerCase();
        newUser.isSystemUser = false;
        if (vcc === 'system')
            newUser.isSystemUser = true;
        newUser.createdDate = null;
        newUser.createdByUserId = null;
        newUser.lastModifiedDate = nowUtc;
        newUser.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetUser(vcc, userId);
        }).then(function (resultCurrentUser) {
            if (!resultCurrentUser)
                throw 'USER_SERVER_ERROR_CANNOT_GET';
            if (!newUser || resultCurrentUser.id !== newUser.id)
                throw 'USER_SERVER_ERROR_ON_MATCH';
            newUser.role = resultCurrentUser.role;
            newUser.limitClients = resultCurrentUser.limitClients;
            return Promise.resolve(true);
        }).then(function (resultDeleteUser) {
            if (!resultDeleteUser)
                throw 'USER_SERVER_ERROR_ON_SAVE';
            return SaveUser(vcc, newUser);
        }).then(function (resultSaveUser) {
            if (!resultSaveUser)
                throw 'USER_SERVER_ERROR_ON_SAVE';
            return redis_service_1.Redis.SaveUser(vcc, newUser);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'USER_SERVER_ERROR_ON_SAVE'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/validate_user_pin', function (req, res, next) {
        logger.info('[UserService::validate_user_pin] Getting req.method:', req.method, 'req.path:', req.path);
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var pin = req.body.pin;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var apiUser = new api_user_implementation_1.ApiUserImplementation();
            return apiUser.ValidateUserPin(vcc, userId, pin);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result ? result.data : null });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UserService::/api/user/validate_user_pin] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.UserService = UserService;
function GetUser(vcc, id) {
    id = (id || '').toLowerCase();
    var cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetUser(vcc, id)
        .then(function (redisUser) {
        if (redisUser) {
            return Promise.resolve(redisUser);
        }
        else {
            logger.info('El dato de GetUser para \'' + id + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetUser(vcc, id);
        }
    }).then(function (user) {
        if (user) {
            user.id = (user.id || '').toLowerCase();
            if (cacheUpdateNeeded)
                redis_service_1.Redis.SaveUser(vcc, user);
            return Promise.resolve(user);
        }
        else {
            throw 'USER_SERVER_ERROR_NOT_FOUND';
        }
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetUser: ' + errMsg);
    });
}
exports.GetUser = GetUser;
function SaveUser(vcc, user) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveUser(vcc, user.id, user.email, user.firstname, user.lastname, user.password, user.role, user.country, user.language, user.timeZone, user.preferences, user.isSystemUser, user.limitClients, user.createdDate, user.createdByUserId, user.lastModifiedDate, user.lastModifiedByUserId));
    if (user.userClients) {
        user.userClients.forEach(function (userClient) {
            promiseArray.push(mssql_service_1.MSSql.SaveUserClient(vcc, userClient.userId, userClient.clientId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    })
        .then(function (result) {
        if (user.role.toLowerCase() === 'auditor' || user.role.toLowerCase() === 'quality_auditor') {
            logger.info('SaveAuditor: ', user);
            return mssql_service_1.MSSql.SaveAuditor(vcc, user.id, '', user.createdDate, user.createdByUserId, user.lastModifiedDate, user.lastModifiedByUserId);
        }
        else {
            return true;
        }
    }).catch(function (err) {
        return Promise.resolve(false);
    });
}
exports.SaveUser = SaveUser;
function ExecuteSaveUserByAPIi6(vcc, user) {
    return mssql_service_1.MSSql.GetAPIUserOCC(vcc)
        .then(function (resultUserAPI) {
        if (!resultUserAPI)
            return Promise.resolve(null);
        return http_request_occ_1.Login(resultUserAPI.user + '@' + resultUserAPI.vcc, resultUserAPI.password);
    })
        .then(function (resultLogin) {
        if (!resultLogin.accessToken)
            return Promise.resolve(null);
        var userToSave = {};
        userToSave.Password = user.password;
        userToSave.isNew = true;
        userToSave.Id = user.id;
        userToSave.Name = user.firstname;
        userToSave.FirstSurname = user.lastname;
        userToSave.UserGroup = (user.role === 'quality_auditor' ? 'qualityauditor' : user.role);
        userToSave.TimeZoneId = 'UYT';
        userToSave.countryCode = '598';
        return http_request_occ_1.SaveUserByAPIi6(resultLogin.accessToken, userToSave);
    }).then(function (resultSaveUserByAPIi6) {
        return Promise.resolve(resultSaveUserByAPIi6);
    }).catch(function (err) {
        logger.error('ExecuteTask (inConcert): Error: ' + err.message);
        return Promise.resolve(null);
    });
}
function SaveUserIntegration(vcc, newUser, ip, flag, isEdit) {
    var agentExist = false;
    var roleExist = false;
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.GetUser(vcc, newUser.id);
    }).then(function (resultUser) {
        if (!isEdit) {
            if (resultUser)
                throw 'USER_ID_ALREADY_EXISTS';
        }
        if (isEdit) {
            newUser.password = newUser.password === null ? resultUser.password !== undefined ? resultUser.password : '' : newUser.password;
            if (!newUser || resultUser.id !== newUser.id)
                throw 'USER_NOT_MATCH';
        }
        logger.info('[UserService::SaveUserIntegration] Not password arrived, using: ' + newUser.password);
        if ((newUser.role === 'quality_agent' || newUser.role === 'quality_auditor') && (integration_config_1.TYPE === 'OCC' || integration_config_1.TYPE === 'INTEGRATED')) {
            return ExecuteSaveUserByAPIi6(vcc, newUser);
        }
        else if (newUser.role === 'quality_agent' && integration_config_1.TYPE === '5.5') {
            return mssql_service_1.MSSql.GetAllegroAgent(integration_config_1.IP_BD, vcc, newUser.id, flag);
        }
        else if (newUser.role === 'quality_auditor' && integration_config_1.TYPE === '5.5') {
            return true;
        }
        else {
            return true;
        }
    }).then(function (result) {
        if (newUser.role === 'quality_agent' && integration_config_1.TYPE === '5.5') {
            agentExist = result ? true : false;
            return mssql_service_1.MSSql.GetAllegroRole(integration_config_1.IP_BD, vcc, newUser.role, flag);
        }
        else if (newUser.role === 'quality_auditor' && integration_config_1.TYPE === '5.5') {
            return true;
        }
        else {
            return true;
        }
    })
        .then(function (result) {
        if ((newUser.role === 'quality_agent' || newUser.role === 'quality_auditor') && integration_config_1.TYPE === '5.5') {
            roleExist = result ? true : false;
            if (agentExist || result) {
                var name_1 = newUser.firstname + (newUser.lastname ? ' ' + newUser.lastname : '');
                return mssql_service_1.MSSql.SaveAllegroUser(newUser.role === 'quality_auditor' ? 'qualityauditor' : newUser.role, 0, '598', newUser.id, 'ENABLED', '0', name_1, 'es', 'UYT', newUser.password, vcc, newUser.role === 'quality_auditor' ? 'system' : vcc, integration_config_1.IP_BD, flag);
            }
            else {
                throw 'ALLEGRO_AGENT_NOT_FOUND';
            }
        }
        else {
            return true;
        }
    }).then(function (result) {
        if (newUser.role === 'quality_agent' && integration_config_1.TYPE === '5.5') {
            if (roleExist) {
                return true;
            }
            return agentExist;
        }
        else if (newUser.role === 'quality_auditor' && integration_config_1.TYPE === '5.5') {
            return Promise.resolve(result);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        if (result) {
            return SaveUser(vcc, newUser);
        }
        return Promise.resolve(false);
    }).then(function (resultSaveUser) {
        if ((newUser.role === 'quality_agent' || newUser.role === 'quality_auditor') && integration_config_1.TYPE === '5.5') {
            if (!resultSaveUser)
                throw 'USER_SAVE_FAILED_QUALITY';
        }
        if (!resultSaveUser)
            throw 'USER_SAVE_FAILED';
        return redis_service_1.Redis.SaveUser(vcc, newUser);
    }).catch(function (err) {
        logger.error('[UserService::SaveUserIntegration] Getting error:', err, 'IP:', ip, 'flag:', flag, 'newUser:', newUser);
        return Promise.reject(err);
    });
}
function SetError(userId, reason) {
    var details = null;
    if ((userId && userId.length) && (reason && reason.length)) {
        details = {};
        details.error = {
            userId: userId,
            reason: reason
        };
    }
    return Promise.resolve(details);
}
function DeleteUser(vcc, userId) {
    var user, reason = '';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        if (!userId) {
            reason = 'MISSING_USER_ID_PARAMETER';
            throw reason;
        }
        return mssql_service_1.MSSql.GetApiUser(vcc, userId);
    })
        .then(function (result) {
        if (result)
            throw 'ERROR_IS_API_USER';
        return mssql_service_1.MSSql.GetUser(vcc, userId);
    }).then(function (result) {
        user = result;
        if (!result) {
            reason = 'ERROR_USER_NOT_FOUND';
            throw reason;
        }
        if (result.role === 'quality_auditor') {
            return mssql_service_1.MSSql.GetAuditorsInteractions(vcc, 'id', userId);
        }
        else {
            return true;
        }
    }).then(function (result) {
        if (result && result[0] && result[0].uncompleteInteractions && result[0].uncompleteInteractions > 0) {
            reason = 'ERROR_USER_PENDING_EVALUATIONS_MSG';
            throw reason;
        }
        if (result) {
            return mssql_service_1.MSSql.DeleteUser(vcc, userId);
        }
        return true;
    }).then(function (result) {
        if (!result) {
            reason = 'ERROR_DELETING_USER';
            throw reason;
        }
        return redis_service_1.Redis.DeleteUser(vcc, user);
    }).then(function (result) {
        if (!result) {
            reason = 'ERROR_DELETING_USER_REDIS';
            throw reason;
        }
        return Promise.resolve(result);
    }).catch(function (error) {
        return SetError(userId, error);
    });
}
