"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiI6GetInteractionFile = exports.ApiI6GetInteractionParts = exports.ApiI6GetInteractionData = exports.ApiI6ConfiguredChannels = exports.ApiI6GetCampaignDispositions = exports.ApiI6SearchCampaigns = exports.ApiI6SearchUsers = exports.SaveUserByAPIi6 = exports.SearchUsers = exports.Login = void 0;
var log4js = require("log4js");
var integration_config_1 = require("../config/integration.config");
var request_shared_1 = require("./request.shared");
var loggerAllegro = log4js.getLogger('ServerScripts');
function Login(user, password) {
    loggerAllegro.info('[http-request-occ::Login] Login por API i6', JSON.stringify({ user: user, password: password }));
    var data = { user: user, password: password };
    var postOptions = {
        uri: "http://" + integration_config_1.DNS + "/inconcert/api/login/",
        body: data,
        headers: {
            'Content-Type': 'application/json'
        }
    };
    return request_shared_1.SimplePostRequest(postOptions)
        .then(function (response) {
        var status = (response && response.status) ? true : false;
        var message = (response && response) ? response : '';
        return Promise.resolve({ status: status, accessToken: message.token });
    })
        .catch(function (err) {
        loggerAllegro.error('[http-request-occ::Login] Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.Login = Login;
function SearchUsers(accessToken, request) {
    loggerAllegro.info('[http-request-occ::SearchUsers] Request (inConcert): Listado de usuarios por API i6');
    var postOptions = {
        uri: "http://" + integration_config_1.DNS + "/inconcert/api/users/search/",
        body: request,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        }
    };
    return request_shared_1.SimplePostRequest(postOptions)
        .then(function (response) {
        return Promise.resolve(response);
    })
        .catch(function (err) {
        loggerAllegro.error('[http-request-occ::SearchUsers] : Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SearchUsers = SearchUsers;
function SaveUserByAPIi6(accessToken, user) {
    loggerAllegro.info('[http-request-occ::SaveUserByAPIi6] Request (inConcert): Creación de usuario por API i6');
    var postOptions = {
        uri: "http://" + integration_config_1.DNS + "/inconcert/api/users/save/",
        body: user,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        }
    };
    return request_shared_1.SimplePostRequest(postOptions)
        .then(function (response) {
        return Promise.resolve(response);
    })
        .catch(function (err) {
        loggerAllegro.error('[http-request-occ::SaveUserByAPIi6] Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SaveUserByAPIi6 = SaveUserByAPIi6;
function ApiI6SearchUsers(vcc, campaignId) {
    loggerAllegro.info('[http-request-occ::ApiI6SearchUsers] Getting users');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://" + integration_config_1.DNS + "/inconcert/api/users/search/",
            body: {
                query: { Application: "interaction" }
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            }
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then(function (response) {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_AGENTS_FAILED';
        var agents = [];
        response['rows'].map(function (r) {
            agents.push({
                vcc: vcc,
                campaignId: null,
                userId: r.Id,
                userName: (r.Name ? r.Name.trim() + ' ' : '') + (r.FirstSurname ? r.FirstSurname.trim() : '')
            });
        });
        return Promise.resolve({ status: true, body: agents });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6SearchUsers] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6SearchUsers = ApiI6SearchUsers;
function ApiI6SearchCampaigns(vcc) {
    loggerAllegro.info('[http-request-occ::ApiI6SearchCampaigns] Getting campaigns');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://" + integration_config_1.DNS + "/inconcert/api/campaigns/search/",
            body: {
                columns: ["Id", "Name", "Description", "dispositionCodes"]
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            }
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then(function (response) {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
        var result;
        result = response['rows'].map(function (r) {
            var campaign = r.Id.split('@');
            var dispositionCodes = [];
            if (r.dispositionCodes) {
                r.dispositionCodes.map(function (dc) {
                    if (dc.childs) {
                        dc.childs.map(function (c1) {
                            if (c1.childs) {
                                c1.childs.map(function (c2) {
                                    if (c2.childs) {
                                        c2.childs.map(function (c3) {
                                            dispositionCodes.push({ parent: dc.name, child: c1.name, child2: c2.name, child3: c3.name });
                                        });
                                    }
                                    else {
                                        dispositionCodes.push({ parent: dc.name, child: c1.name, child2: c2.name, child3: null });
                                    }
                                });
                            }
                            else {
                                dispositionCodes.push({ parent: dc.name, child: c1.name, child2: null, child3: null });
                            }
                        });
                    }
                    else {
                        dispositionCodes.push({ parent: dc.name, child: null, child2: null, child3: null });
                    }
                });
            }
            return { id: campaign[0], description: r.Description, dispositionCodes: dispositionCodes };
        });
        return Promise.resolve({ status: true, body: result });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6SearchCampaigns] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6SearchCampaigns = ApiI6SearchCampaigns;
function ApiI6GetCampaignDispositions(vcc) {
    loggerAllegro.info('[http-request-occ::ApiI6GetCampaignDispositions] Getting campaigns');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/campaigns/get_dispositions/' + null,
            method: 'GET',
            body: {
                columns: ["Id", "Name", "Description", "dispositionCodes"]
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            }
        };
        return request_shared_1.SimpleGetRequest(postOptions);
    })
        .then(function (response) {
        return Promise.resolve({ status: true, body: response });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetCampaignDispositions] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetCampaignDispositions = ApiI6GetCampaignDispositions;
function ApiI6ConfiguredChannels(vcc, campaign) {
    loggerAllegro.info('[http-request-occ::ApiI6ConfiguredChannels] Getting campaigns');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: "http://" + integration_config_1.DNS + "/inconcert/api/campaigns/get_configured_channels/",
            body: {
                campaign: campaign
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            }
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then(function (response) {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_TYPES_FAILED';
        var keys = Object.keys(response.channels);
        var result;
        result = keys.map(function (k) {
            return { intance: vcc, id: k, description: k };
        });
        return Promise.resolve({ status: true, body: result });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6ConfiguredChannels] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6ConfiguredChannels = ApiI6ConfiguredChannels;
function ApiI6GetInteractionData(vcc, interactionId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractionData] Getting interaction parts');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var getOptions = {
            uri: "http://" + integration_config_1.DNS + "/inconcert/api/interactions/get/" + interactionId,
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            }
        };
        return request_shared_1.SimpleGetRequest(getOptions);
    })
        .then(function (response) {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_DATA_FAILED';
        return Promise.resolve({ status: true, body: response.interaction });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractionData] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionData = ApiI6GetInteractionData;
function ApiI6GetInteractionParts(vcc, interactionId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractionParts] Getting interaction parts');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var getOptions = {
            uri: "http://" + integration_config_1.DNS + "/inconcert/api/cinteractions/media/list/" + interactionId,
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            }
        };
        return request_shared_1.SimpleGetRequest(getOptions);
    })
        .then(function (response) {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_PARTS_FAILED';
        return Promise.resolve({ status: true, body: response.media });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractionParts] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionParts = ApiI6GetInteractionParts;
function ApiI6GetInteractionFile(vcc, interactionId, section, part) {
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractionFile] Getting campaigns');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var getOptions = {
            uri: "http://" + integration_config_1.DNS + "/inconcert/api/interactions/media/get/?interaction=" + interactionId + "&id=" + interactionId + "&section=" + section + "&part=" + part,
            headers: {
                'Authorization': result.accessToken
            },
            responseType: 'stream'
        };
        return request_shared_1.SimpleGetRequest(getOptions);
    })
        .then(function (result) {
        loggerAllegro.info('[http-request-occ::ApiI6GetInteractionFile] OK');
        var buffer = null;
        var bufs = [];
        return new Promise(function (resolve, reject) {
            result.on('error', function (err) {
                loggerAllegro.error('[http-request-occ::ApiI6GetInteractionFile] SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
            });
            result.once('close', function () {
                try {
                    buffer = Buffer.concat(bufs);
                    return resolve(buffer);
                }
                catch (e) {
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                }
            });
            result.on('data', function (d) {
                try {
                    bufs.push(d);
                }
                catch (e) {
                    return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                }
            });
        });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractionFile] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionFile = ApiI6GetInteractionFile;
