"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleDeleteRequest = exports.SimplePutRequest = exports.SimplePostRequest = exports.SimpleGetRequest = exports.AttachedFilePostRequest = void 0;
var log4js = require("log4js");
var axios_1 = require("axios");
var http = require('http');
var https = require('https');
var logger = log4js.getLogger('ServerScripts');
function SimpleResult(rawResponse) {
    var requestResponse = null;
    if (typeof rawResponse === 'string') {
        try {
            requestResponse = JSON.parse(rawResponse);
        }
        catch (err) {
            requestResponse = null;
        }
    }
    else {
        requestResponse = rawResponse;
    }
    return requestResponse;
}
function AttachedFilePostRequest(data, formData) {
    var requestConfig = {
        method: 'POST',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: {
            'Content-Type': "multipart/form-data; boundary=" + formData._boundary,
            'Connection': 'keep-alive'
        },
        maxContentLength: Infinity,
        maxBodyLength: Infinity,
        data: formData
    };
    return axios_1.default(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return requestResponse.data;
        }
        else {
            throw 'SERVER_ERROR_ATTACHED_FILE_POST_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::AttachedFileRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.AttachedFilePostRequest = AttachedFilePostRequest;
function SimpleGetRequest(data) {
    var requestConfig = {
        method: 'GET',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers
    };
    if (data.responseType)
        requestConfig.responseType = data.responseType;
    return axios_1.default(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_GET_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimpleGetRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimpleGetRequest = SimpleGetRequest;
function SimplePostRequest(data) {
    var requestConfig = {
        method: 'POST',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    return axios_1.default(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_POST_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimplePostRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimplePostRequest = SimplePostRequest;
function SimplePutRequest(data) {
    var requestConfig = {
        method: 'PUT',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    return axios_1.default(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_DELETE_PUT_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimplePutRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimplePutRequest = SimplePutRequest;
function SimpleDeleteRequest(data) {
    var requestConfig = {
        method: 'DELETE',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    return axios_1.default(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_DELETE_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimpleDeleteRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimpleDeleteRequest = SimpleDeleteRequest;
