"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var express = require("express");
var moment = require("moment");
var log4js = require("log4js");
var ab2str = require('arraybuffer-to-string');
var md5 = require('md5');
var mssql_config_1 = require("../config/mssql.config");
var mssql_service_1 = require("../db/mssql.service");
var mssql_init_1 = require("../db/mssql.init");
var idatha_service_1 = require("../services/idatha.service");
var cassandra_service_1 = require("../db/cassandra.service");
var cassandra_init_1 = require("../db/cassandra.init");
var integration_config_1 = require("../config/integration.config");
var config_1 = require("../config/config");
var user_service_1 = require("../services/user.service");
var category_service_1 = require("../services/category.service");
var role_service_1 = require("../services/role.service");
var vcc_service_1 = require("../services/vcc.service");
var feeling_service_1 = require("../services/feeling.service");
var neuro_linguistic_programming_service_1 = require("../services/neuro-linguistic-programming.service");
var environment_service_1 = require("../services/environment.service");
var api_shared_1 = require("../shared/api.shared");
var loggerAllegro = log4js.getLogger('Task');
function ExecuteTask() {
    loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de VCCs');
    var VCCs = [];
    var vcc = '';
    var description = '';
    var nowUtc = moment().utc().toDate();
    var obji, objj, leni, lenj;
    if (integration_config_1.TYPE === 'OCC') {
        loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de VCCs de i6');
        return cassandra_service_1.CassandraSql.GetCassandraVCCs()
            .then(function (result) {
            VCCs = result;
            return Promise.resolve(VCCs);
        }).then(function (result) {
            if (result === [])
                return Promise.resolve(null);
            return ExecuteCreateVCC(result);
        }).catch(function (err) {
            loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
            return Promise.resolve(null);
        });
    }
    else if (integration_config_1.TYPE === '5.5' || integration_config_1.TYPE === 'INTEGRATED') {
        loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de VCCs con 5.5/Integrado');
        var flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
        return mssql_service_1.MSSql.GetIntegrationVCCs(integration_config_1.IP_BD, flag, integration_config_1.TYPE)
            .then(function (result) {
            if (result === [])
                return Promise.resolve(null);
            return ExecuteCreateVCC(result);
        }).catch(function (err) {
            loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
            return Promise.resolve(null);
        });
    }
    else {
        loggerAllegro.error('ExecuteTask (inConcert): no cumplio con el tipo de ambiente');
        return Promise.resolve(null);
    }
}
function ExecuteCreateVCC(vccs) {
    var promiseArray = [];
    if (vccs) {
        vccs.forEach(function (vcc) {
            promiseArray.push(CreateVCC(vcc.id, vcc));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function CreateVCC(vcc, objVCC) {
    var nowUtc = moment().utc().toDate();
    var api;
    loggerAllegro.info('[vcc-bulk.task::CreateVCC] Checking if exists vcc ' + vcc);
    return mssql_service_1.MSSql.GetVCC(vcc)
        .then(function (resultVCC) {
        if (resultVCC)
            throw 'Ya existe un vcc con el identificador \'' + vcc + '\'';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] The vcc name is available');
        return vcc_service_1.SaveVCC(vcc, objVCC);
    }).then(function (resultSaveVCC) {
        if (!resultSaveVCC)
            throw 'Se produjo un error al guardar el vcc';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting the GetApi for system');
        return api_shared_1.GetAPIDataFromConfig('system');
    }).then(function (result) {
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Receiving the system\'s info');
        api = {
            'speechmatics': result.find(function (x) {
                return x.apiName === 'SpeechMatics';
            }), 'idatha': result.find(function (x) {
                return x.apiName === 'IDATHA';
            })
        };
        return Promise.resolve(api);
    }).then(function (resultGetAPI) {
        if (!resultGetAPI)
            throw 'Se produjo un error al recuperar datos del api';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Registering on idatha');
        var userName = config_1.REALM_USER_NAME;
        var password = config_1.REALM_PASSWORD;
        var idathaAuth = { vcc: vcc, userName: userName, password: password, realm: vcc };
        return idatha_service_1.IdathaRegister(resultGetAPI.idatha, idathaAuth);
    }).then(function (resultIdathaRegister) {
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] ExecuteTask (inConcert): accessToken: ' + resultIdathaRegister);
        var user = {};
        user.id = 'analytic';
        user.isSystemUser = false;
        user.role = 'administrator';
        user.firstname = 'user' + objVCC.id;
        user.password = md5('analytic' + 'inc0nc3rt');
        user.createdDate = nowUtc;
        user.createdByUserId = 'admin';
        user.lastModifiedDate = nowUtc;
        user.lastModifiedByUserId = 'admin';
        user.language = null;
        return user_service_1.SaveUser(vcc, user);
    }).then(function (resultSaveUser) {
        if (!resultSaveUser)
            throw 'USER_SERVER_ERROR_ON_SAVE';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting roles from system');
        return mssql_service_1.MSSql.GetRoles('system', 'name', '');
    }).then(function (resultGetRoles) {
        if (!resultGetRoles)
            throw 'Se produjo un error al recuperar roles';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Registering roles for new vcc');
        return role_service_1.SaveRoles(vcc, resultGetRoles);
    }).then(function (resultSaveRoles) {
        if (!resultSaveRoles)
            throw 'Se produjo un error al guardar los roles';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting role for system');
        return mssql_service_1.MSSql.GetRole('system', null);
    }).then(function (resultGetRole) {
        if (!resultGetRole)
            throw 'Se produjo un error al recuperar permisos y rutas';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving roles routes');
        return role_service_1.SaveRolesRoutes(vcc, resultGetRole);
    }).then(function (resultSaveRolesRoutes) {
        if (!resultSaveRolesRoutes)
            throw 'Se produjo un error al guardar los permisos y rutas';
        return mssql_service_1.MSSql.GetRolePermissionsGroups('system');
    }).then(function (resultGetRolePermissionsGroups) {
        if (!resultGetRolePermissionsGroups)
            throw 'Se produjo un error al recuperar grupos de rol';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving role perission groups for new vcc');
        return role_service_1.SaveRolePermissionsGroup(vcc, resultGetRolePermissionsGroups);
    }).then(function (resultSaveRolePermissionsGroup) {
        if (!resultSaveRolePermissionsGroup)
            throw 'Se produjo un error al guardar permisos de grupo';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting feelings from system');
        return mssql_service_1.MSSql.GetFeelings('system', 'name', '');
    }).then(function (resultGetFeelings) {
        if (!resultGetFeelings)
            throw 'Se produjo un error al recuperar tipos de sentimientos';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving feelings for new vcc');
        return feeling_service_1.SaveFeelings(vcc, resultGetFeelings);
    }).then(function (resultSaveFeeling) {
        if (!resultSaveFeeling)
            throw 'Se produjo un error al guardar tipos de sentimientos';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting NeuroLinguisticProgrammings from system');
        return mssql_service_1.MSSql.GetNeuroLinguisticProgrammings('system', 'name', '');
    }).then(function (resultGetNeuroLinguisticProgrammings) {
        if (!resultGetNeuroLinguisticProgrammings)
            throw 'Se produjo un error al recuperar los tipos de pnl';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving NeuroLinguisticProgrammings for new vcc');
        return neuro_linguistic_programming_service_1.SaveNeuroLinguisticProgrammings(vcc, resultGetNeuroLinguisticProgrammings);
    }).then(function (resultSaveNeuroLinguisticProgramming) {
        if (!resultSaveNeuroLinguisticProgramming)
            throw 'Se produjo un error al guardar tipos de pnl';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting words from system');
        return mssql_service_1.MSSql.GetWords('system', 'NeuroLinguisticProgrammingId', '', 'init', 'es');
    }).then(function (resultGetWords) {
        if (!resultGetWords)
            throw 'Se produjo un error al recuperar las palabras de pnl';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving words for new vcc');
        return neuro_linguistic_programming_service_1.SaveWord(vcc, resultGetWords);
    }).then(function (resultSaveWord) {
        if (!resultSaveWord)
            throw 'Se produjo un error al guardar las palabras de pnl';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting category\'s names from system');
        return mssql_service_1.MSSql.GetCategories('system', 'name', '');
    }).then(function (resultGetCategories) {
        if (!resultGetCategories)
            throw 'Se produjo un error al recuperar las categorías';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving category\'s names for new vcc');
        return category_service_1.SaveCategories(vcc, resultGetCategories);
    }).then(function (resultSaveCategories) {
        if (!resultSaveCategories)
            throw 'Se produjo un error al guardar las categorias';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting Environments from system');
        return mssql_service_1.MSSql.GetEnvironments('system', 'name', '');
    }).then(function (resultGetEnvironments) {
        if (!resultGetEnvironments)
            throw 'Se produjo un error al recuperar los ambientes';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving Environments for new vcc');
        return environment_service_1.SaveEnvironments(vcc, resultGetEnvironments);
    }).then(function (resultSaveEnvironments) {
        if (!resultSaveEnvironments)
            throw 'Se produjo un error al guardar las categorías';
        if (integration_config_1.TYPE === '5.5') {
            var flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
            loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving allegro group for new vcc type 5.5');
            return mssql_service_1.MSSql.SaveAllegroGroup(vcc, 'quality_agent', 0, 'quality_agent', integration_config_1.IP_BD, flag);
        }
        else {
            loggerAllegro.info('[vcc-bulk.task::CreateVCC] vcc type isn\'t 5.5, continuing');
            return Promise.resolve(true);
        }
    }).then(function (resultSaveAllegroGroup) {
        if (!resultSaveAllegroGroup)
            throw 'Se produjo un error al guardar el rol quality_agent en allegro';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Finished create process for vcc ' + vcc);
        return Promise.resolve(true);
    }).catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        loggerAllegro.error('[vcc-bulk.task::CreateVCC] Error procesing \'' + err + '\': ' + errorMsg);
        return Promise.resolve(false);
    });
}
if (require.main === module) {
    var app_1 = express();
    var sqlConn_1;
    var cassandraConn_1;
    mssql_config_1.MSSQL_CONFIG.connectionTimeout = 300 * 1000;
    mssql_config_1.MSSQL_CONFIG.requestTimeout = 300 * 1000;
    mssql_init_1.MsSqlInit(app_1)
        .then(function (result) {
        sqlConn_1 = app_1.get('sqlConn');
        if (integration_config_1.TYPE === 'OCC')
            return cassandra_init_1.CassandralInit(app_1);
        return Promise.resolve(true);
    }).then(function (result) {
        if (integration_config_1.TYPE === 'OCC')
            cassandraConn_1 = app_1.get('cassandraConn');
        return ExecuteTask();
    }).then(function (results) {
        loggerAllegro.info('Task (allegro): Termino la sincronizacion de VCCs');
        sqlConn_1.close();
        if (integration_config_1.TYPE === 'OCC')
            cassandraConn_1.close();
        setTimeout(function () {
            process.exit(0);
        }, 200);
    }).catch(function (error) {
        loggerAllegro.error('Task (allegro): Termino la sincronizacion de VCCs con errores :', error);
        sqlConn_1.close();
        if (integration_config_1.TYPE === 'OCC')
            cassandraConn_1.close();
        setTimeout(function () {
            process.exit(0);
        }, 200);
    });
}
