"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisImplementation = void 0;
var log4js = require("log4js");
var moment = require("moment");
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var file_manager_1 = require("../shared/file-manager");
var idatha_model_1 = require("../models/idatha.model");
var scp2 = require('scp2');
var billing_implementation_1 = require("../implementation/billing.implementation");
var idatha_service_1 = require("../services/idatha.service");
var cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
var request_shared_1 = require("../shared/request.shared");
var billing_batch_model_1 = require("../models/billing-batch.model");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'SemanticAnalysisImplementation';
var SemanticAnalysisImplementation = (function () {
    function SemanticAnalysisImplementation() {
        this.errorData = new billing_batch_model_1.BatchError();
    }
    SemanticAnalysisImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_SEMANTIC_ANALYSIS;
        var executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(function (result) {
            return redis_service_1.Redis.LRange('SemanticAnalysis', 0, ratio - 1);
        }).then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.SemanticAnalysisExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('SemanticAnalysis', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticAnalysisImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisImplementation.prototype.SemanticAnalysisExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var isCancel = false;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var batchTypeName = '';
        var batchTypeId = '';
        return redis_service_1.Redis.RPopLPush('SemanticAnalysis', 'SemanticAnalysisExecute')
            .then(function (queueId) {
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (processInteractionResult) {
            if (processInteractionResult) {
                logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1');
                processInteraction = processInteractionResult;
                if (processInteraction.statusDescription === 'pendingAnalysis') {
                    return _this.SendJSON(processInteraction, currentQueueId, executionCode);
                }
                else if (processInteraction.statusDescription === 'pendingDownloadJSON') {
                    var validTime = moment().format('x');
                    validTime = validTime * 1;
                    var nextExecutionTime = processInteraction.uploadedDate;
                    nextExecutionTime = nextExecutionTime * 1 + config_1.NEXT_EXECUTION_TIME_SEMANTIC_ANALYSIS;
                    if (validTime > nextExecutionTime) {
                        return _this.DownloadJSON(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_EXECUTION', nextExecutionTime, 'CURRENT_TIME', validTime);
                    }
                }
            }
            if (!processInteraction && currentQueueId)
                logger.warn('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
            return null;
        })
            .then(function (result) {
            if (result)
                processInteraction = result;
            return mssql_service_1.MSSql.GetEvaluationPlanByProcessId(vcc, processId);
        })
            .then(function (result) {
            if (result && result.id) {
                batchTypeName = 'EVALUATION_PLAN';
                batchTypeId = result.id;
            }
            else {
                batchTypeName = 'TRANSCRIPTION';
                batchTypeId = processInteraction.processId;
            }
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        })
            .then(function (result) {
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result && processInteraction && processInteraction.status === 5 && processInteraction.statusDescription === 'ready') {
                logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidChar', batchTypeName, batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidWords', batchTypeName, batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'TotalPhrases', batchTypeName, batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'CharsPerPhrase', batchTypeName, batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'IdathaApi', batchTypeName, batchTypeId);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId, 1, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else if (result && processInteraction && processInteraction.status === 5 && processInteraction.statusDescription === 'pendingDownloadJSON' && processInteraction.retry < config_1.RETRY_SEMANTIC_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('SemanticAnalysis', currentQueueId, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS * config_1.RATIO_SEMANTIC_ANALYSIS * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else if (result && processInteraction && processInteraction.status === -5 && processInteraction.statusDescription === 'pendingRemove') {
                logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_EMPTY_CONTENT');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId, 0, 'REMOVED_BY_EMPTY_CONTENT');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            var promiseArray = [];
            if (processInteraction
                && processInteraction.status === 5
                && processInteraction.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            else if (processInteraction
                && (processInteraction.retry >= config_1.RETRY_SEMANTIC_ANALYSIS
                    ||
                        processInteraction.errorMessage === 'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY')) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchTypeId, batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] Catching error: ' + errorMessage);
            logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisImplementation.prototype.SendJSON = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var path = config_1.TRANSCRIPTION_FOLDER;
        var jsonContent = null;
        var idatha = {
            user: config_1.IDATHA_USER,
            password: config_1.IDATHA_PASSWORD,
            ip: config_1.IDATHA_IP,
            httpPort: config_1.IDATHA_HTTP_PORT,
            sshPort: config_1.IDATHA_SSH_PORT,
        };
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetUserAccessTokenByRealm(interaction.vcc, interaction.vcc);
        })
            .then(function (result) {
            var promises = [];
            if (result && result.accessToken) {
                promises.push(Promise.resolve(true));
            }
            else {
                promises.push(Promise.resolve(false));
                var auth = new idatha_model_1.IdathaAuth(interaction.vcc, idatha.user, idatha.password, interaction.vcc);
                promises.push(idatha_service_1.IdathaRegister(idatha, auth));
            }
            return Promise.all(promises);
        })
            .then(function (results) {
            var found = results[0];
            if (found) {
                return true;
            }
            var result = results[1];
            if (!result && result.length < 1) {
                throw 'COULD_NOT_CREATE_REALM';
            }
            return true;
        })
            .then(function (_) {
            var regex = /\.(mp3|wav)$/;
            var jsonFileName = interaction.fileName.replace(regex, '.json');
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return file_manager_1.ReadFileContent(path, 'utf-8');
        }).then(function (result) {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                jsonContent.filterdata = {
                    start_date: new Date(interaction.startDate),
                    end_date: new Date(interaction.endDate),
                    channel: interaction.interactionType,
                    campaign: interaction.campaign,
                    disposition: interaction.dispositionCode,
                    agent: interaction.agent,
                    origin: interaction.origin
                };
            }
            catch (ex) {
                jsonContent = '';
                throw 'ERROR_FILE_JSON';
            }
            return JSON.stringify(jsonContent);
        }).then(function (result) {
            return file_manager_1.WriteFileContent(path, result, 'utf-8', true);
        }).then(function (_) {
            if (jsonContent && jsonContent.results && jsonContent.results.length > 0) {
                return _this.IdathaSendJSONFiles(idatha, interaction, currentQueueId, executionCode);
            }
            else {
                return 'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY';
            }
        }).then(function (result) {
            if (typeof result === 'boolean' && result === true) {
                interaction['status'] = 5;
                interaction['statusDescription'] = 'pendingDownloadJSON';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                _this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisImplementation::SendJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'JSON_TRANSCRIPTION_WAS_NOT_TRANSFERRED');
                interaction['status'] = -5;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                if (typeof result === 'string' && result === 'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY') {
                    interaction['statusDescription'] = 'pendingRemove';
                    interaction['errorMessage'] = 'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY';
                }
                else {
                    interaction['errorMessage'] = 'JSON_TRANSCRIPTION_WAS_NOT_TRANSFERRED';
                }
                _this.errorData.SetErrorData('SendJSON', interaction['errorMessage'], interaction['errorMessage']);
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::SendJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, errorMessage);
            interaction['status'] = -5;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('SendJSON', errorMessage, err);
            return Promise.resolve(interaction);
        });
    };
    SemanticAnalysisImplementation.prototype.IdathaCreateDefaultDirIfNotExists = function (client, path) {
        return new Promise(function (resolve, reject) {
            client.mkdir(path, 777, function (err) {
                if (err) {
                    logger.error('[SemanticAnalysisImplementation::IdathaCreateDefaultDir]', err);
                    reject(err);
                }
                else {
                    resolve(true);
                }
            });
            client.on("error", function (err) {
                logger.error('[SemanticAnalysisImplementation::IdathaCreateDefaultDir]', err);
                reject(err);
            });
        });
    };
    SemanticAnalysisImplementation.prototype.IdathaUploadFile = function (client, src, destination) {
        return new Promise(function (resolve, reject) {
            client.upload(src, destination, function (err) {
                if (err) {
                    logger.error('[SemanticAnalysisImplementation::IdathaUploadFile]', err);
                    reject(err);
                }
                else {
                    resolve(true);
                }
            });
            client.on("error", function (err) {
                logger.error('[SemanticAnalysisImplementation::IdathaUploadFile]', err);
                reject(err);
            });
        });
    };
    SemanticAnalysisImplementation.prototype.IdathaSendJSONFiles = function (idatha, processInteraction, currentQueueId, executionCode) {
        var _this = this;
        var client = new scp2.Client({
            port: idatha.sshPort,
            host: idatha.ip,
            username: idatha.user,
            password: idatha.password
        });
        var regex = /\.(mp3|wav)$/;
        var src = config_1.TRANSCRIPTION_FOLDER + '/' + processInteraction.vcc.toLowerCase() + '/' + processInteraction.fileName.replace(regex, '.json');
        var destination = '/home/' + idatha.user + '/in_concert_speech_analytics/input/' + processInteraction.vcc.toLowerCase() + '/default/';
        return promises_shared_1.DummyPromise().then(function (_) {
            return _this.IdathaCreateDefaultDirIfNotExists(client, destination);
        }).then(function (result) {
            if (result) {
                return _this.IdathaUploadFile(client, src, destination);
            }
            return result;
        }).then(function (result) {
            client.close();
            if (result) {
                _this.errorData.CleanErrorData();
            }
            return result;
        }).catch(function (err) {
            if (client) {
                client.close();
            }
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::IdathaSendJSONFiles] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            _this.errorData.SetErrorData('IdathaSendJSONFiles', errorMessage, err);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisImplementation.prototype.DownloadJSON = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var path = '';
        var jsonContent = null;
        var data;
        var idatha = {
            user: config_1.IDATHA_USER,
            password: config_1.IDATHA_PASSWORD,
            ip: config_1.IDATHA_IP,
            httpPort: config_1.IDATHA_HTTP_PORT,
            sshPort: config_1.IDATHA_SSH_PORT,
        };
        var cognitiveService;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(interaction.vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            var promiseArray = [];
            promiseArray.push(file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_FOLDER + '/' + interaction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER + '/' + interaction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.COGNITIVE_ANALYSIS_FOLDER + '/' + interaction.vcc));
            return Promise.all(promiseArray);
        }).then(function (result) {
            var regex = /\.(mp3|wav|json)$/;
            var jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.SEMANCTIC_ANALYSIS_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            var key = interaction.vcc + ':' + interaction.vcc;
            return idatha_service_1.IdathaLogin(idatha, interaction.vcc);
        }).then(function (userAccessToken) {
            if (!userAccessToken)
                throw 'NOT_FOUND_ACCESS_TOKEN';
            var regex = /\.(mp3|wav|json)$/;
            var fileName = interaction.fileName.replace(regex, '');
            return _this.IdathaDocumentWords(idatha, userAccessToken.accessToken, interaction.id, interaction.section, interaction.part, currentQueueId, executionCode, fileName);
        }).then(function (result) {
            data = result;
            logger.info('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting path:', path);
            if (result) {
                var jsonData = JSON.parse(data);
                var validChar_1 = 0;
                var validWords = 0;
                if (jsonData && jsonData.words && jsonData.words.length && jsonData.words.length > 0) {
                    validWords = jsonData.words.length;
                    jsonData.words.map(function (r) {
                        validChar_1 += r.name ? r.name.length + 1 : 0;
                    });
                }
                interaction['validChar'] = validChar_1;
                interaction['validWords'] = validWords;
                interaction['idathaApi'] = config_1.IDATHA_IP + ':' + config_1.IDATHA_HTTP_PORT;
                return file_manager_1.WriteFileContent(path, data, 'utf-8');
            }
            return null;
        }).then(function (result) {
            if (result) {
                var cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
                var transcription = cognitiveAnalysis.SortJson(JSON.parse(data));
                var phrases = cognitiveAnalysis.SplitTranscriptionContentInPhrases(cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 100, transcription);
                logger.info('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases:', phrases.length);
                return phrases;
            }
            return null;
        }).then(function (result) {
            var phrases;
            if (result && result.length) {
                interaction['charsPerPhrase'] = cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 0;
                interaction['totalPhrases'] = result.length;
                var regex = /\.(mp3|wav|json)$/;
                var jsonFileName = interaction.fileName.replace(regex, '.json');
                var path_1 = config_1.COGNITIVE_ANALYSIS_FOLDER;
                path_1 += '/';
                path_1 += interaction.vcc;
                path_1 += '/';
                path_1 += jsonFileName;
                phrases = { phrases: result };
                var stringPhrases = JSON.stringify(phrases);
                return file_manager_1.WriteFileContent(path_1, stringPhrases, 'utf-8');
            }
            return null;
        }).then(function (result) {
            if (result) {
                logger.info('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting succes: ready');
                interaction['status'] = 5;
                interaction['statusDescription'] = 'ready';
                interaction['isTranscribed'] = 1;
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                _this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: JSON_IDATHA_IS_NOT_FOUND transcription content:', data, 'path:', path);
                interaction['status'] = -5;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = 'JSON_IDATHA_IS_NOT_FOUND';
                _this.errorData.SetErrorData('DownloadJSON', interaction['errorMessage'], interaction['errorMessage']);
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -5;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            _this.errorData.SetErrorData('DownloadJSON', errorMessage, err);
            return Promise.resolve(interaction);
        });
    };
    SemanticAnalysisImplementation.prototype.IdathaDocumentWords = function (idatha, accessToken, audioId, section, part, currentQueueId, executionCode, fileName) {
        var _this = this;
        var url = "http://" + idatha.ip + ":" + idatha.httpPort + "/api/query/document/" + fileName;
        var options = {
            uri: url,
            headers: {
                'Authorization': 'Bearer ' + accessToken
            }
        };
        var data = '';
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return request_shared_1.SimpleGetRequest(options);
        }).then(function (response) {
            try {
                var regex = /\bNaN\b/g;
                data = JSON.stringify(response);
                data = data.replace(regex, 'null');
                _this.errorData.CleanErrorData();
                return Promise.resolve(data);
            }
            catch (err) {
                logger.error('[SemanticAnalysisImplementation::IdathaDocumentWords] executionCode:', executionCode, 'getting queueId:', currentQueueId, err, 'options:', options);
                _this.errorData.SetErrorData('IdathaDocumentWords', 'ERROR', err);
                return Promise.resolve(null);
            }
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::IdathaDocumentWords] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, 'options:', options);
            _this.errorData.SetErrorData('IdathaDocumentWords', errorMessage, err);
            return Promise.resolve(null);
        });
    };
    SemanticAnalysisImplementation.prototype.SemanticAnalysisFileGenerate = function () {
        return null;
    };
    SemanticAnalysisImplementation.prototype.SemanticAnalysisViewFileGenerate = function (interaction) {
        var _this = this;
        var semanticAnalysisFolder = '';
        var semanticAnalysisFilePath = '';
        var semanticAnalysisViewFolder = '';
        var semanticAnalysisViewFileName = '';
        var semanticAnalysisViewFilePath = '';
        var semanticAnalysisViewFileContent = null;
        semanticAnalysisFolder = config_1.SEMANCTIC_ANALYSIS_FOLDER;
        semanticAnalysisFolder += '/';
        semanticAnalysisFolder += interaction.vcc;
        semanticAnalysisViewFolder = config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER;
        semanticAnalysisViewFolder += '/';
        semanticAnalysisViewFolder += interaction.vcc;
        semanticAnalysisViewFileName = interaction.jsonFileName;
        semanticAnalysisFilePath = semanticAnalysisFolder + '/' + semanticAnalysisViewFileName;
        semanticAnalysisViewFilePath = semanticAnalysisViewFolder + '/' + semanticAnalysisViewFileName;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return file_manager_1.ExistInFileSystem(semanticAnalysisFilePath);
        })
            .then(function (result) {
            if (result) {
                return file_manager_1.CreateFolder(semanticAnalysisViewFolder);
            }
            else
                throw 'SERVER_ERROR_SEMANTIC_ANALYSIS_FILE_NOT_FOUND';
        }).then(function (result) {
            return file_manager_1.ReadFileContent(semanticAnalysisFilePath, 'utf-8');
        }).then(function (result) {
            if (result) {
                var semanticAnalysisFileContent = JSON.parse(result);
                var cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
                semanticAnalysisViewFileContent = cognitiveAnalysis.SortJson(semanticAnalysisFileContent);
                return file_manager_1.WriteFileContent(semanticAnalysisViewFilePath, JSON.stringify(semanticAnalysisViewFileContent), 'utf-8');
            }
            else
                throw 'SERVER_ERROR_SEMANTIC_ANALYSIS_FILE_FAILED';
        })
            .then(function (result) {
            _this.errorData.CleanErrorData();
            return {
                fileName: semanticAnalysisViewFileName,
                transcriptionResult: semanticAnalysisViewFileContent
            };
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticAnalysisImplementation::SemanticAnalysisViewFileGenerate] Getting error:', errorMessage);
            _this.errorData.SetErrorData('SemanticAnalysisViewFileGenerate', errorMessage, error);
            return Promise.resolve(false);
        });
    };
    SemanticAnalysisImplementation.prototype.CognitiveAnalysisViewFileGenerate = function () {
        return null;
    };
    return SemanticAnalysisImplementation;
}());
exports.SemanticAnalysisImplementation = SemanticAnalysisImplementation;
