"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AgentsService = void 0;
var log4js = require("log4js");
var promises_shared_1 = require("../shared/promises.shared");
var mssql_service_1 = require("../db/mssql.service");
var allegro_service_1 = require("./allegro.service");
var logger = log4js.getLogger('ServerScripts');
function AgentsService(app) {
    app.get('/api/agent/get_agents?', function (req, res, next) {
        logger.info('[AgentsService::get_agents] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var agents = [];
        var agentStr = req.query.agents || '';
        var campaign = req.query.campaign || null;
        logger.info('[AgentsService::get_agents] Received vcc::agents::campaigns => ' + vcc, '::', agentStr, '::', campaign);
        var agentRole = 'quality_agent';
        var isAgent = res.locals.userRole.id === agentRole;
        if (isAgent) {
            logger.info('[AgentsService::get_agents] Agent role quality_agent.');
            agentStr = res.locals.userId;
        }
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return allegro_service_1.GetAgents(vcc, type, campaign);
        })
            .then(function (result) {
            agents = result;
            if (isAgent)
                agents = agents.filter(function (a) { return a.userId === agentStr; });
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: agents });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/agent/evaluation', function (req, res, next) {
        logger.info('[AgentsService::evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var referrer = res.locals.pageReferrer.split('/').pop();
        var searchType = null;
        var campaigns = req.body.campaigns ? req.body.campaigns : [];
        switch (referrer) {
            case 'auditorinbox':
                searchType = 'PENDING';
                break;
            case 'evaluation':
                searchType = 'HISTORY';
                break;
            case 'evaluation-agent':
                searchType = 'HISTORY';
                break;
            case 'evaluation-admin':
                searchType = 'HISTORY';
                break;
        }
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAgentsFromEvaluation(vcc, userId, campaigns, searchType);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AGENTS_SEARCH_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AgentsService::evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/agent/get_agent_qualification_average/:agentId', function (req, res, next) {
        logger.info('[AgentsService::get_agent_qualification_average] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var agentStr = req.params.agentId || '';
        logger.info('[AgentsService::get_agent_qualification_average] Received instance ' + instance);
        logger.info('[AgentsService::get_agent_qualification_average] Received agent ' + agentStr);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[AgentsService::get_agent_qualification_average] Calling GetAgentQualificationAverage SP');
            return mssql_service_1.MSSql.GetAgentQualificationAverage(instance, agentStr);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.debug('[AgentsService::get_agent_qualification_average] Received result from GetAgentQualificationAverage SP: ' + JSON.stringify(result));
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[AgentsService::get_agent_qualification_average] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/agent/interaction/', function (req, res) {
        logger.info('[NLUService::agent_interaction] Receives ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise().then(function (_) {
            return mssql_service_1.MSSql.GetInteractionAgents(vcc);
        }).then(function (agents) {
            res.json({ status: true, description: 'ok', data: agents });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::agent_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AgentsService = AgentsService;
