"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionTypeService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var http_request_occ_1 = require("../shared/http-request-occ");
var logger = log4js.getLogger('ServerScripts');
function InteractionTypeService(app) {
    app.get('/api/interaciontype/get_interactiontypes?', function (req, res, next) {
        logger.info('[InteractionTypeService::get_interactiontypes] Received ' + req.method + ' method for \'' + req.path + '\'');
        var instance = res.locals.instance;
        var campaign = req.query.campaign || null;
        logger.info('[InteractionTypeService::get_interactiontypes] Requesting interactionTypes for vcc: ' + instance);
        logger.info('[InteractionTypeService::get_interactiontypes] Requesting interactionTypes for campaigns: ' + campaign);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return http_request_occ_1.ApiI6ConfiguredChannels(instance, campaign);
        })
            .then(function (result) {
            if (result && result.status === true) {
                return Promise.resolve(result.body);
            }
            else {
                logger.info('[InteractionTypeService::get_interactiontypes] Requesting interactionTypes to speechanalytic, instance: ' + instance);
                return mssql_service_1.MSSql.GetInteractionTypes(instance, null, null);
            }
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionTypeService::get_interactiontypes] Received interactionTypes for instance: ' + instance);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[InteractionTypeService::get_interactiontypes] Error \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
}
exports.InteractionTypeService = InteractionTypeService;
