"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveCategoryPermission = exports.SaveRolePermissionsGroup = exports.SaveRolesRoutes = exports.SaveRoles = exports.SaveRole = exports.GetRole = exports.RoleService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function RoleService(app) {
    app.get('/api/role/get_roles/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetRoles(vcc, search_criteria, search_expression);
        }).then(function (resultRoles) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultRoles });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/role/get_role/:role_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var roleId = req.params.role_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!roleId)
                throw 'ROLE_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetRole(vcc, roleId);
        }).then(function (resultRole) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultRole });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/role/create_role', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newRole = (req.body || {});
        newRole.createdDate = nowUtc;
        newRole.createdByUserId = userId;
        newRole.lastModifiedDate = nowUtc;
        newRole.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetRole(vcc, newRole.id);
        }).then(function (resultRole) {
            if (resultRole)
                'ROLE_SERVER_ERROR_EXISTS';
            return SaveRole(vcc, newRole);
        }).then(function (resultSaveRole) {
            if (!resultSaveRole)
                throw 'ROLE_SERVER_ERROR_ON_SAVE';
            return redis_service_1.Redis.DeleteRole(vcc, newRole);
        }).then(function (resultSaveRole) {
            if (res.headersSent)
                return;
            logger.info('Rol \'' + newRole.id + '\' creado correctamente');
            res.json({ status: resultSaveRole, description: (resultSaveRole ? 'SERVER_SUCCESS_MESSAGE' : 'ROLE_SERVER_ERROR_ON_SAVE'), data: resultSaveRole });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/role/save_role', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newRole = (req.body || {});
        newRole.createdDate = null;
        newRole.createdByUserId = null;
        newRole.lastModifiedDate = nowUtc;
        newRole.lastModifiedByUserId = userId;
        logger.info('[RoleService::/api/role/save_role] getting newRole:', newRole);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.DeleteRolePermission(vcc, newRole.id);
        })
            .then(function (result) {
            logger.info('[RoleService::/api/role/save_role] deleted permissions :', result);
            if (!result)
                throw 'CANNOT_DELETE_PERMISSION_ROLE';
            return mssql_service_1.MSSql.GetRole(vcc, newRole.id);
        }).then(function (resultGetRole) {
            logger.info('[RoleService::/api/role/save_role] getting resultGetRole:', resultGetRole);
            if (!resultGetRole)
                throw 'CANNOT_GET_ROLE_DATA';
            return SaveRole(vcc, newRole);
        }).then(function (resultSaveRole) {
            if (!resultSaveRole)
                throw 'ROLE_SAVE_FAILED';
            return redis_service_1.Redis.DeleteRole(vcc, newRole);
        }).then(function (resultSaveRole) {
            logger.info('[UserService::/api/role/save_user] getting resultSaveRole:', resultSaveRole, 'after Redis.DeleteRole');
            if (res.headersSent)
                return;
            logger.info('Rol \'' + newRole.id + '\' guardado correctamente');
            res.json({ status: resultSaveRole, description: (resultSaveRole ? 'SERVER_SUCCESS_MESSAGE' : 'ROLE_SAVE_FAILED'), data: resultSaveRole });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/role/delete_role', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = req.body.role;
        var roleId = role.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!role)
                throw 'ROLE_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteRole(vcc, roleId);
        }).then(function (resultDeleteRole) {
            if (!resultDeleteRole)
                throw 'ROLE_SERVER_ERROR_ON_DELETE';
            return redis_service_1.Redis.DeleteRole(vcc, role);
        }).then(function (resultDeleteRole) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteRole, description: (resultDeleteRole ? 'SERVER_SUCCESS_MESSAGE' : 'ROLE_SERVER_ERROR_ON_DELETE'), data: resultDeleteRole });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/role/delete_roles', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var roles = req.body.roles || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!roles)
                throw 'ROLE_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            roles.forEach(function (role) {
                promiseArray.push(mssql_service_1.MSSql.DeleteRole(vcc, role.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteRoles) {
            var result;
            if (resultDeleteRoles.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Roles eliminados correctamente');
                roles.forEach(function (role) {
                    redis_service_1.Redis.DeleteRole(vcc, role);
                });
            }
            return result;
        }).then(function (resultDeleteRoles) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteRoles, description: (resultDeleteRoles ? 'SERVER_SUCCESS_MESSAGE' : 'ROLE_SERVER_ERROR_ON_DELETE'), data: resultDeleteRoles });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/role/get_roles_permission_groups', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetRolePermissionsGroups(vcc);
        }).then(function (resultPermissionGroups) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultPermissionGroups });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/role/get_category_permission', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCategoryPermissions(vcc);
        }).then(function (resultCategoryPermission) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultCategoryPermission });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.RoleService = RoleService;
function GetRole(vcc, id) {
    var cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetRole(vcc, id)
        .then(function (redisRole) {
        if (redisRole) {
            return Promise.resolve(redisRole);
        }
        else {
            logger.info('El dato de GetRole para \'' + id + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetRole(vcc, id);
        }
    }).then(function (role) {
        if (role) {
            if (cacheUpdateNeeded)
                redis_service_1.Redis.SaveRole(vcc, role);
            return Promise.resolve(role);
        }
        else {
            throw 'ROLE_SERVER_ERROR_NOT_FOUND';
        }
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetRole: ' + errMsg);
    });
}
exports.GetRole = GetRole;
function SaveRole(vcc, role) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveRole(vcc, role.id, role.description, role.systemRole, role.type, role.createdDate, role.createdByUserId, role.lastModifiedDate, role.lastModifiedByUserId));
    if (role.permissions) {
        role.permissions.forEach(function (rolePermission) {
            promiseArray.push(mssql_service_1.MSSql.SaveRolePermission(vcc, role.id, rolePermission.groupId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveRole = SaveRole;
function SaveRoles(vcc, role) {
    var promiseArray = [];
    if (role) {
        role.forEach(function (roles) {
            promiseArray.push(mssql_service_1.MSSql.SaveRole(vcc, roles.id, roles.description, roles.systemRole, roles.type, roles.createdDate, roles.createdByUserId, roles.lastModifiedDate, roles.lastModifiedByUserId));
            if (roles.permissions) {
                roles.permissions.forEach(function (rolePermission) {
                    promiseArray.push(mssql_service_1.MSSql.SaveRolePermission(vcc, rolePermission.roleId, rolePermission.groupId));
                });
            }
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveRoles = SaveRoles;
function SaveRolesRoutes(vcc, role) {
    var promiseArray = [];
    if (role.permissions) {
        role.permissions.forEach(function (rolePermission) {
            promiseArray.push(mssql_service_1.MSSql.SaveRolePermission(vcc, rolePermission.roleId, rolePermission.groupId));
        });
    }
    if (role.routePermissions) {
        role.routePermissions.forEach(function (roleRoutePermissions) {
            promiseArray.push(mssql_service_1.MSSql.SaveRoutePermission(vcc, roleRoutePermissions.routeId, roleRoutePermissions.groupId, roleRoutePermissions.route, roleRoutePermissions.level));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveRolesRoutes = SaveRolesRoutes;
function SaveRolePermissionsGroup(vcc, permissionsGroup) {
    var promiseArray = [];
    if (permissionsGroup) {
        permissionsGroup.forEach(function (rolePermissionsGroups) {
            promiseArray.push(mssql_service_1.MSSql.SaveRolePermissionsGroup(vcc, rolePermissionsGroups.groupId, rolePermissionsGroups.category, rolePermissionsGroups.type));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveRolePermissionsGroup = SaveRolePermissionsGroup;
function SaveCategoryPermission(vcc, categories) {
    var promiseArray = [];
    if (categories) {
        categories.forEach(function (category) {
            promiseArray.push(mssql_service_1.MSSql.SaveCategoryPermissions(vcc, category.category, category.position));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveCategoryPermission = SaveCategoryPermission;
